      SUBROUTINE GAUSSQ(KIND, N, ALPHA, BETA, KPTS, ENDPTS, B, T, W)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE

C***      This set of routines computes the nodes X(I) and weights
C***   C(I) for Gaussian-type quadrature rules with pre-assigned
C***   nodes.  These are used when one wishes to approximate
C***
C***            Integral (From A to B)  F(X) W(X) DX
C***
C***                         N
C***   by                   SUM C  F(X )
C***                        I=1  I    I
C***
C***   here W(X) is one of six possible non-negative weight
C***   functions (listed below), and F(X) is the
C***   function to be integrated.  Gaussian quadrature is particularly
C***   useful on infinite intervals (with appropriate weight
C***   functions), since then other techniques often fail.
C***
C***      Associated with each weight function W(X) is a set of
C***   orthogonal polynomials.  The nodes X(I) are just the zeroes
C***   of the proper N-th degree polynomial.
C***
C***Input Parameters:
C***
C***   KIND     An integer between 1 and 6 giving the type of
C***            quadrature rule.
C***
C***   KIND = 1=  Legendre quadrature, W(X) = 1 on (-1, 1).
C***   KIND = 2=  Chebyshev quadrature of the first kind
C***              W(X) = 1/SQRT(1 - X*X) on (-1, +1).
C***   KIND = 3=  Chebyshev quadrature of the second kind
C***              W(X) = SQRT(1 - X*X) on (-1, 1).
C***   KIND = 4=  Hermite quadrature, W(X) = EXP(-X*X) on
C***              (-INFINITY, +INFINITY).
C***   KIND = 5=  Jacobi quadrature, W(X) = (1-X)**ALPHA * (1+X)**
C***              BETA on (-1, 1), ALPHA, BETA .GT. -1.
C***              Note: KIND=2 and 3 are a special case of this.
C***   KIND = 6=  Generalized Laguerre quadrature, W(X) = EXP(-X)*
C***              X**ALPHA on (0, +INFINITY), ALPHA .GT. -1.
C***
C***   N        The number of points used for the quadrature rule.
C***   ALPHA    DOUBLE PRECISION parameter used only for Gauss-Jacobi and
C***            Gauss-Laguerre quadrature (otherwise use 0.).
C***   BETA     DOUBLE PRECISION parameter used only for Gauss-Jacobi
C***            quadrature--(otherwise use 0.).
C***   KPTS     (INTEGER) normally 0, unless the left or right end-
C***            point (or both) of the interval is required to be a
C***            node (this is called Gauss-Radau or Gauss-Lobatto
C***            quadrature).  Then KPTS is the number of fixed
C***            endpoints (1 OR 2).
C***   ENDPTS   DOUBLE PRECISION array of length 2.  Contains the values of
C***            any fixed endpoints, IF KPTS = 1 or 2.
C***   B        DOUBLE PRECISION scratch array of length N.
C***
C***Output Parameters (both arrays of length N):
C***
C***   T        will contain the desired nodes X(1),,,X(N).
C***   W        will contain the desired weights C(1),,,C(N).
C***
C***Subroutines required:
C***
C***   GBSLVE, CLASS, and GBTQL2 are provided. Underflow may sometimes
C***   occur, but it is harmless if the underflow interrupts are
C***   turned off as they are on this machine.
C***
C***Accuracy:
C***
C***   The routine was tested up to N = 512 for Legendre quadrature,
C***   up to N = 136 for Hermite, up to N = 68 for Laguerre, and up
C***   to N = 10 or 20 in other cases.  In all but two instances,
C***   comparison with tables in Ref. 3 showed 12 or more significant
C***   digits of accuracy.  The two exceptions were the weights for
C***   Hermite and Laguerre quadrature, where underflow caused somE
C***   very small weights to be set to zero.  This is, of course,
C***   completely harmless.
C***
C***Method:
C***
C***      The coefficients of the three-term recurrence relation
C***   for the corresponding set of orthogonal polynomials are
C***   used to form a symmetric tridiagonal matrix, whose
C***   eigenvalues (determined by the implicit QL-method with
C***   shifts) are just the desired nodes.  The first components of
C***   the orthonormalized eigenvectors, when properly scaled,
C***   yield the weights.  This technique is much faster than using a
C***   root-finder to locate the zeroes of the orthogonal polynomial.
C***   For further details, see Ref. 1.  Ref. 2 contains details of
C***   Gauss-Radau and Gauss-Lobatto quadrature only.
C***
C***References:
C***
C***   1.  Golub, G. H., and WELSCH, J. H.,  Calculation of Gaussian
C***       quadrature rules,  Mathematics of Computation 23 (April,
C***       1969), pp. 221-230.
C***   2.  Golub, G. H.,  Some Modified Matrix Eigenvalue Problems,
C***       SIAM Review 15 (April, 1973), pp. 318-334 (Section 7).
C***   3.  Stroud and Secrest, Gaussian Quadrature Formulas, Prentice-
C***       Hall, Englewood Cliffs, N.J., 1966.
C***
C***..................................................................
C****
      DOUBLE PRECISION MUZERO
      DIMENSION B(N),T(N),W(N),ENDPTS(2)

      CALL CLASS (KIND,N,ALPHA,BETA,B,T,MUZERO)

C***  The matrix of coefficients is assumed to be symmetric.
C***The array T contains the diagonal elements, the array
C***B the off-diagonal elements.
C***Make appropriate changes in the lower right 2 by 2
C***submatrix.

      IF (KPTS.EQ.0)  GO TO 100
      IF (KPTS.EQ.2)  GO TO  50

C***If KPTS=1, only T(N) must be changed.

      T(N) =GBSLVE(ENDPTS(1), N, T, B)*B(N-1)**2 + ENDPTS(1)
      GO TO 100

C***If KPTS=2, T(N) and B(N-1) must be recomputed.

   50 GAM =GBSLVE(ENDPTS(1),N,T,B)
      T1 = ((ENDPTS(1) - ENDPTS(2))/(GBSLVE(ENDPTS(2),N,T,B) - GAM))
      B(N-1) = DSQRT(T1)
      T(N) = ENDPTS(1) + GAM*T1

C***Note that the indices of the elements of B run from 1 to N-1
C***and thus the value of B(N) is arbitrary.
C***Now compute the eigenvalues of the symmetric tridiagonal
C***matrix, which has been modified as necessary.
C***The method used is a QL-type method with origin shifting.

  100 W(1) = 1.0D0
      DO 105 I = 2, N
  105    W(I) = 0.0D0

      CALL GBTQL2 (N,T,B,W,IERR)
      DO 110 I = 1, N
  110    W(I) = MUZERO * W(I) * W(I)

      RETURN
      END
      FUNCTION GBSLVE(SHIFT,N,A,B)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

C***   This procedure performs elimination to solve for the
C***N-th component of the solution DELTA to the equation
C***
C***        (JN - SHIFT*IDENTITY) * DELTA  = EN,
C***
C***where EN is the vector of all zeroes except for 1 in
C***the N-th position.
C***
C***   The matrix JN is symmetric tridiagonal, with diagonal
C***elements A(I), off-diagonal elements B(I).  This equation
C***must be solved to obtain the appropriate changes in the lower
C***2 by 2 submatrix of coefficients for orthogonal polynomials.
C***

      DIMENSION  A(N),B(N)

      ALPHA = A(1) - SHIFT
      NM1 = N - 1
      DO 10 I = 2, NM1
   10    ALPHA = A(I) - SHIFT - B(I-1)**2/ALPHA
      GBSLVE = 1.0D0  /ALPHA
      RETURN
      END
      SUBROUTINE CLASS(KIND,N,ALPHA,BETA,B,A,MUZERO)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL DGAMMA
      SAVE

C***  This procedure supplies the coefficients A(J), B(J) of the
C***recurrence relation

C***        B P (X) = (X - A ) P   (X) - B   P   (X)
C***         J J            J   J-1       J-1 J-2
C***
C***for the various classical (normalized) orthogonal polynomials,
C***and the zero-th moment
C***
C***        MUZERO = INTEGRAL W(X) DX
C***
C***of the given polynomial weight function W(X).  Since the
C***polynomials are orthonormalized, the tridiagonal matrix is
C***guaranteed to be symmetric.
C***
C***  The input parameter ALPHA is used only for Laguerre and
C***Jacobi polynomials, and the parameter BETA is used only for
C***Jacobi polynomials.  The Laguerre and Jacobi polynomials
C***require the GAMMA function.
C***
C***..................................................................

      DIMENSION A(N),B(N)
      DOUBLE PRECISION MUZERO
      DATA PI / 3.141592653589793D0  /

      NM1 = N - 1
      GO TO (10,20,30,40,50,60),KIND

C***  KIND = 1=  Legendre polynomials P(X)
C***  on (-1, +1), W(X) = 1.

   10 MUZERO = 2.0D0
      DO 11 I = 1, NM1
         A(I) = 0.0D0
         ABI = I
   11    B(I) = ABI/DSQRT(4*ABI*ABI - 1.0D0  )
      A(N) = 0.0D0
      RETURN

C***  KIND = 2=  Chebyshev polynomials of the first kind T(X)
C***  on (-1, +1), W(X) = 1 /SQRT(1 - X*X).

   20 MUZERO = PI
      DO 21 I = 1, NM1
         A(I) = 0.0D0
   21    B(I) = 0.5D0
      B(1) = DSQRT(0.5D0  )
      A(N) = 0.0D0
      RETURN

C***  KIND = 3=  Chebyshev polynomials of the second kind U(X)
C***  on (-1, +1), W(X) = SQRT(1 - X*X).

   30 MUZERO = PI/2.0D0
      DO 31 I = 1, NM1
         A(I) = 0.0D0
   31    B(I) = 0.5D0
      A(N) = 0.0D0
      RETURN

C***  KIND = 4=  Hermite polynomials H(X) on (-INFINITY,
C*** +INFINITY), W(X) = EXP(-X**2).

   40 MUZERO = DSQRT(PI)
      DO 41 I = 1, NM1
         A(I) = 0.0D0
   41    B(I) = DSQRT(I/2.0D0  )
      A(N) = 0.0D0
      RETURN

C***  KIND = 5=  Jacobi polynomials P(ALPHA, BETA)(X) on
C***  (-1, +1), W(X) = (1-X)**ALPHA + (1+X)**BETA, ALPHA and
C***  BETA greater than -1.

   50 AB = ALPHA + BETA
      ABI = 2.0D0   + AB
      MUZERO = 2.0D0   ** (AB + 1.0D0  ) * DGAMMA(ALPHA + 1.0D0  ) * DGA
     VMMA(
     X BETA + 1.0D0  ) / DGAMMA(ABI)
      A(1) = (BETA - ALPHA)/ABI
      B(1) = DSQRT(4.0D0  *(1.0D0   + ALPHA)*(1.0D0   + BETA)/((ABI + 1.
     V0D0  )*
     1  ABI*ABI))
      A2B2 = BETA*BETA - ALPHA*ALPHA
      DO 51 I = 2, NM1
         ABI = 2.0D0  *I + AB
         A(I) = A2B2/((ABI - 2.0D0  )*ABI)
   51    B(I) = DSQRT (4.0D0  *I*(I + ALPHA)*(I + BETA)*(I + AB)/
     1   ((ABI*ABI - 1)*ABI*ABI))
      ABI = 2.0D0  *N + AB
      A(N) = A2B2/((ABI - 2.0D0  )*ABI)
      RETURN

C***  KIND = 6=  Laguerre polynomials L(ALPHA)(X) on
C***  (0, +INFINITY), W(X) = EXP(-X) * X**ALPHA, ALPHA greater
C***  than -1.

   60 MUZERO = DGAMMA(ALPHA + 1.0D0  )
      DO 61 I = 1, NM1
         A(I) = 2.0D0  *I - 1.0D0   + ALPHA
   61    B(I) = DSQRT(I*(I + ALPHA))
      A(N) = 2.0D0  *N - 1 + ALPHA
      RETURN
      END
      SUBROUTINE GBTQL2(N,D,E,Z,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE

C***   This subroutine is a translation of the ALGOL procedure IMTQL2,
C***Num. Math. 12, 377-383(1968) by Martin and Wilkinson,
C***as modified in Num. Math. 15, 450(1970) by Dubrulle.
C***Handbook For Auto. Comp., VOL.II-Linear Algebra, 241-248(1971).
C***
C***This subroutine finds the eigenvalues and first components of the
C***eigenvectors of a symmetric tridiagonal matrix by the implicit QL
C***method, and is adapted from the EISPAK routine IMTQL2.
C***
C***On Input=
C***
C***   N is the order of the matrix;
C***
C***   D contains the diagonal elements of the input matrix;
C***
C***   E contains the subdiagonal elements of the input matrix
C***     in its first N-1 positions.  E(N) is arbitrary;
C***
C***   Z contains the first row of the identity matrix.
C***
C*** On Output=
C***
C***   D contains the eigenvalues in ascending order.  If an
C***     error exit is made, the eigenvalues are correct but
C***     unordered for indices 1, 2, ..., IERR-1;
C***
C***   E has been destroyed;
C***
C***   Z contains the first components of the orthonormal eigenvectors
C***     of the symmetric tridiagonal matrix.  If an error exit is
C***     made, Z contains the eigenvectors associated with the stored
C***     eigenvalues;
C***
C***   IERR is set to
C***
C***   IERR is set to
C***     ZERO       for normal return,
C***     J          if the J-th eigenvalue has not been
C***                determined after 30 iterations.
C***
C***------------------------------------------------------------------

      INTEGER I, J, K, L, M, N, II, MML, IERR
      DIMENSION  D(N),E(N),Z(N)
      DOUBLE PRECISION    MACHEP

C***  MACHEP is a machine-dependent parameter specifying
C***the relative precision of floating point arithmetic.
C***MACHEP = 16.0D0**(-13) for long form arithmetic
C***on S360.

       MACHEP=1.0D-14

      IERR = 0
      IF (N .EQ. 1) GO TO 1001

      E(N) = 0.0D0
      DO 240 L = 1, N
         J = 0
C***Look for small sub-diagonal element.

  105    DO 110 M = L, N
            IF (M .EQ. N) GO TO 120
            IF ( ABS(E(M)) .LE. MACHEP * ( ABS(D(M)) +  ABS(D(M+1))))
     X         GO TO 120
  110    CONTINUE

  120    P = D(L)
         IF (M .EQ. L) GO TO 240
         IF (J .EQ. 30) GO TO 1000
         J = J + 1

C***Form shift.

         G = (D(L+1) - P) / (2.0D0   * E(L))
         R = DSQRT(G*G+1.0D0  )
         G = D(M) - P + E(L) / (G +  DSIGN(R, G))
         S = 1.0D0
         C = 1.0D0
         P = 0.0D0
         MML = M - L

C***For I=M-1 step -1 untiL L do.

         DO 200 II = 1, MML
            I = M - II
            F = S * E(I)
            B = C * E(I)
            IF ( ABS(F) .LT.  ABS(G)) GO TO 150
            C = G / F
            R = DSQRT(C*C+1.0D0  )
            E(I+1) = F * R
            S = 1.0D0   / R
            C = C * S
            GO TO 160
  150       S = F / G
            R = DSQRT(S*S+1.0D0  )
            E(I+1) = G * R
            C = 1.0D0   / R
            S = S * C
  160       G = D(I+1) - P
            R = (D(I) - G) * S + 2.0D0   * C * B
            P = S * R
            D(I+1) = G + P
            G = C * R - B

C***Form first component of vector.

            F = Z(I+1)
            Z(I+1) = S * Z(I) + C * F
            Z(I) = C * Z(I) - S * F

  200    CONTINUE

         D(L) = D(L) - P
         E(L) = G
         E(M) = 0.0D0
         GO TO 105
  240 CONTINUE

C***Order eigenvalues and eigenvectors.

      DO 300 II = 2, N
         I = II - 1
         K = I
         P = D(I)

         DO 260 J = II, N
            IF (D(J) .GE. P) GO TO 260
            K = J
            P = D(J)
  260    CONTINUE

         IF (K .EQ. I) GO TO 300
         D(K) = D(I)
         D(I) = P

         P = Z(I)
         Z(I) = Z(K)
         Z(K) = P

  300 CONTINUE

      GO TO 1001

C***Set error -- no convergence to an
C***eigenvalue after 30 iterations.

 1000 IERR = L
 1001 RETURN

C***Last card of GBTQL2.

      END
