C**********************************************************************
C                           *****RMPROP*****
C                              VERSION 2.1.2
C                    This version uses Full R matrix
C***********************************************************************
C**   This program is a multichannel quantum scattering program using
C**   the method of J.C.Light and R.B.walker, J. CHEM. PHYS. 65, 4272 (1976).
C***********************************************************************
C   Correspondence between variable names in the Light and Walker paper
C(referred to as paper 1) and those in this program is as follows:
C             *                                             *
C Name of     *   Name of equivalent array or variable in   *Equation(s)
C array or    *   paper 1 and brief description of it       *in paper 1
C variable in *                                             *
C this program*                                             *
C or its      *                                             *
C subroutines *                                             *
C***********************************************************************
C PCDMT       *D          Interaction matrix in each sector *  8
C             *           (stored in packed form)           *
C
C UPCDMT      *D          Interaction matrix in each sector *  8
C             *           (stored in unpacked form)         *
C             *                                             *
C EVALI       *LAMBDA**2  Diagonal D                        *  14
C             *                                             *
C LAMOSQ,LAMSQ*           Diagonal D AT R0 and in the       *
C LAMSQE      *           asymptotic region                 *
C             *                                             *
C EFCNI       *T          Matrix which diagonalizes D       *  14
C             *                                             *
C NDIM,ND     *N          Number of terms in the primitive  * 14-17
C             *           basis                             *
C             *                                             *
C PDIM,PD     *P          Number of terms in the contracted * 14-17
C             *           basis                             *
C             *                                             *
C TAU,TINVI   *SCRIPT T   Transformation matrix             * 21,29
C             *                                             *
C P1,P3INV    *PN         The step propagators              *31A-31D
C             *(Capital P subscript N=1,4)                  *
C             *                                             *
C SR1,SR2,SR3,*RN         The sector R matrices             *34A-34D
C SR4         *(Lower case R subscript N=1,4)               *
C             *                                             *
C R1MT,R2MT   *                                             *
C R3MT,R4MT   *RN         The composite sector R matrix     *36A-36D
C             *(Capital R subscript N=1,2,3,4)              *
C             *                                             *
C R1MTM1,     *                                             *
C R2MTM1,     *                                             *
C R3MTM1,     *           The composite sector R matrices   *
C R4MTM1      *           in the previous sector            *
C***********************************************************************
C          Brief description of general structure of program
C              (See *****NAMELIST***** for some details)
C
C        Initial formatted read from FORTRAN unit 5 defines for example
C      the reduced mass, total angular momentum, the size of the basis
C      sets, the initial stepsize, and the various energies and step-
C      sizes at which to calculate.  It also determines the starting
C      and stopping distance for the propagation, selects quantities
C      to be calculated during the asymptotic analysis and determines
C      the number of potential approximations or different sizes of
C      basis set or steps to use during the calculation.
C        A call to subroutine PREPOT then reads in from FORTRAN unit 4
C      information required by the potential including the internal
C      quantum numbers of the collision partners (from which internal
C      energies can be calculated) and symmetry (e.g. in the case of
C      identical molecules) to be used in the calculation.
C        For each potential approximation the program loops through a
C      series of energy values at one stepsize or through several
C      stepsizes at one energy.  Consecutive calculations at several
C      energies permit efficient utilization of that information which
C      needs to be calculated only once, at the reference energy, and
C      which can then be saved and used for subsequent energies with
C      minor changes.  Included in this are calculations of potential
C      elements, D matrix elements, stepsizes, script T matrices, and
C      information used in the diagonalization of the D matrix.
C      Calculations at several energies require energy independent
C      potentials so that the stepsizes will be energy independent.
C      LSTAP is a logical variable read from FORTRAN unit 4 in
C      the potential.  If LSTAP is false after the program completes
C      loops through the energies or stepsizes for one potential the
C      program reads the PREPOT again, followed by POT to define
C      another potential approximation which it treats in the same way
C      as the first. The program continues in this way until it finds
C      LSTAP is true and defines this as the last approximation.
C***********************************************************************
C        Three new features were introduced in the 1/31/78 version of
C      this program which were not suggested in the Light and Walker
C      paper.
C        These features make it possible to:
C
C 1.   Reduce the number of channels propagated from PDIM (the initially
C      defined contracted basis set dimension) to NRED (the new sector
C      dependent number of channels propagated) by dropping closed
C      channels as they become effectively uncoupled from the remaining
C      channels.  The criterion used depends on the magnitude of the
C      elements of the matrix SR2 for the particular closed channel.
C      Note that this will be done only if the input logical variable
C      LR4MT=.TRUE., that is only if just R4MT is propagated.
C
C 2.   Neglect very closed channels in the asymptotic analysis.  The
C      criterion depends on the magnitude of the off diagonal elements
C      of the matrix R4MT for the closed channels considered, when
C      the input logical variable LR4MT=.TRUE., or on the magnitude of
C      the off-diagonal elements of the matrix 'M' which depends on
C      the full R matrix, when the full R matrix is propagated.
C
C 3.   To run at another energy for which the calculations require
C      more sectors to converge than for the reference energy or
C      previous energies.  This is effected by returning to a
C      #reference mode# and calculating in each sector all necessary
C      quantities, e.g. D matrix elements, whenever the number of
C      sectors exceeds the greatest number calculated for any previous
C      energy.  although for reasonably similar energies it should be
C      possible to use the reduced dimensions calculated at the
C      reference energy for other energies, the code does not make that
C      approximation.
C
C        Since the calculation of the inverted script T matrix depends
C      on the dimension of the propagated basis set the code recomputes
C      the script T matrix in each sector for which the reduced
C      dimensionality differs from that of the previous energy.  In
C      order to facilitate this the entire (PDIM,PDIM) script T matrix
C      is calculated at the first energy unless calculations are to be
C      made at only one energy.  In that case only that portion of the
C      script T matrix corresponding to the channels actually propagated
C      in a given sector is calculated.
C
C        If calculations are to be done at several energies, then when
C      the program computes the stepsizes at the reference energy it
C      uses the PDIM basis set rather than the reduced number of
C      channels.  This means that the stepsizes are slightly more
C      conservatively calculated.  The same stepsizes are used at all
C      energies.
C
C        The justification for reducing the number of channels
C      propagated or for neglecting very closed channels in the
C      asymptotic analysis is only understood for examples for which
C      only the R4MT needs to be propagated, i.e. when there is a large
C      repulsive effective potential at the orgin. 
C***********************************************************************
C                         *****Dimensions*****
C      In the descriptions of the dimensions of various arrays given
C      below note the following.
C 1.     ID is defined below.
C 2.     M is any integer greater than or equal to NSECT and
C        NSECT is defined below.
C 3.     NNE is defined below.
C***********************************************************************
C
C          *****Labeled COMMON used by this program*****
C          ***** and by the indicated subroutines  *****
C       ********if the indicated subroutine name is********
C       ********     in parentheses then it is     ********
C       ********           user-supplied           ********
C       ***************************************************
C
C***Arrays must be dimensioned equal to the number in parentheses.
C
C      LCM/LEFACT, LBES, LCONV, LEARLY, LENTER, LR4MT, LSTOP1, LSTOP2
C          Used in-RMPROP, GNSCAT, TITLES.
C
C      PTIMCM/PARTIM(10)
C          Used in-RMPROP, (POT).
C
C      DIMCOM/IDX,NSECTX,NANX,NSAVEX,NVIBX,NVIB1X,NNEX,NBARX
C          Used in-RMPROP, EXSCAT, GNSCAT, TDUMP, TLOAD, TAUMTS,
C                  (PREPOT), (POT).
C
C      TIMCOM/TIMC(50)
C          Used in-RMPROP, GNSCAT.
C
C      PRNTCM/IUNITS, IFTXX
C          Used in-RMPROP, GNSCAT, PRNTSU.
C***********************************************************************
C             *****Dimensions of arrays not in COMMON*****
C             *****     used by this program      ********
C             ********************************************
C***   Dimensions may ge greater than or equal to the variable
C***   in parenthesis: if the parentheses hold an integer the array
C***   is forced to have this dimension
C
C      E(ABS(NE)) :
C        E-array of total energy values given in hartree atomic units
C      energy values in the literature are sometimes the translational
C      energy in the 1st channel, i.e., Elit=E-Einternal(1st channel).
C        NE-number of energies entered. Recall that if several stepsize
C      trials are to be made only one energy can be used.
C      Note that NE should be less than or equal to twenty (20).
C
C      PD(NPD) :
C        PD-Array of the number of terms in the contracted basis set.
C        NPD-number of PD entered.
C      Note that NPD should be less than or equal to TEN (10).
C
C      EPSA(NEPS), EPSB(NEPS), EPSC(NEPS), REPSA(NEPS), REPSB(NEPS) :
C        EPSA, EPSB, EPSC, REPSA, REPSB-real arrays of the stepsize and
C      RABC criteria for the I-th trial stepsize calculation.  The
C      stepsize criteria EPSA(I), EPSB(I) and EPSC(I) are used in the
C      regions RABC.LT.REPSA(I), REPSB(I).GE.RABC.GE.REPSA(I), and
C      RABC.GT.REPSB(I), respectively.
C      Recall that as EPSA, EPSB or EPSC become smaller the stepsize
C      criterion becomes stricter.
C      In addition note that there are minimum stepsizes read in for
C      the above regions: HMINA(NEPS), HMINB(NEPS), and HMINC(NEPS)
C      and maximum stepsizes read in for the above regions: HMAXA(NEPS),
C      HMAXB(NEPS), AND HMAXC(NEPS).  One can fix the stepsize at a
C      given value within a region by setting hmin and hmax for a
C      region to that value.
C        NEPS-number of sets of EPSA, EPSB, and REPS values (that is,
C      the number of trial stepsize calculations).  Recall that if several
C      stepsize trials are to be made only one energy can be used.
C      Recall that NEPS should be less than or equal to five (5).
C
C      LPR(10) : Elements of this array are passed as arguments to
C                various subroutines but not altered there.
C
C      Variable:       If true print:
C        LPR(1)    Execution times.
C        LPR(2)    Asymptotic diagonalized D matrix elements at all
C                  energies.
C        LPR(3)    Not used.
C        LPR(4)    D MATRIX elements in each sector at the reference energy.
C        LPR(5)    Eigenvalues in each sector at all energies.
C        LPR(6)    Eigenfunctions in each sector at the reference energy.
C        LPR(7)    TAUI and TINVI in each sector at the reference energy
C                  and TINVI at other energies in those sectors in which
C                  it must be recalculated.
C        LPR(8)    SMRMT"S in each sector at all energies.
C        LPR(9)    RMT"S in each sector at all energies.
C        LPR(10)   Arrays of rabc and corresponding stepsizes.
C
C      RCUSP(NCUSP) :
C        RCUSP-Positions of cusps in the potential.
C        NCUSP-The number of cusps in the potential.
C      If NCUSP.GE.1, the program forces the right hand boundary of
C      sector which would otherwise include a cusp to lie on the cusp.
C      the stepsizes of the next two sectors are defined differently
C      than usual.  NCUSP should be less than or equal to four (4).
C
C      RPSM(IPMX) :
C      Intermediate distances at which the asymptotic analysis will be
C      performed and at which various quantities will be printed out.
C      Depending on the value of LBGS (see discussion of LGS in gnscat).
C      Usually the S-matrix is printed out here to manually test
C      convergence of the S-matrix with respect to R (center of mass).
C        IPMX-The number of intermediate distances at which the
C      asymptotic analysis should be performed, read in from FORTRAN
C      unit 5.  IPMX should be less than or equal to twenty (20).
C
C      RSTART(ABS(NE))  :
C      RSTART(1) is the starting distance for the first run (original
C      or .NOT.LTYPE2 second energy) and the remainder are the starting
C      distances for LTYPE2 second-energy runs or for multiple basis or
C      multiple orbital angular momentum runs.  Note that for anything
C      except .NOT.LTYPE2 second energy runs all the RSTART(I) should
C      be identical. NE should be less than or equal to twenty (20).
C
C      LAMSQE(ID)  :
C      Diagonal elements of the D-matrix in the asymptotic region
C      where the centrifugal contribution to the D matrix is zero.
C
C      H(NSECT)  :
C      The stepsize in each sector.
C      NSECT is set by a parameter statement in an include file in 
C      beginning of the program.
C
C      NRD(NSECT)  :
C      NRD is the number of channels to be propagated in the current
C      sector for the first energy.  If there is more than one energy
C      it is therefore the maximum number of channels to be propagated
C      in any sector.
C
C      EVALE(ID,NSECT)  :
C      EVALE is an array used to store the eigenvalues of the packed
C      interaction matrix so that they can be used at other energies.
C
C      DMIM1(ID)  :
C      In each sector DMIM1 contains the diagonal D matrix elements
C      from the previous sector, which are used to compute the stepsize
C      for that sector.
C
C      LMB0SQ(ID,NNE)  :
C      Eigenvalues of the interaction potential at the starting point
C      for LTYPE2 second energies.
C
C      EEV(NNE)  :
C      The values of the total energy (energies) in electron volts.
C
C      EVADD(NNE)  :
C      The difference between the first energy and the IE'th energy in
C      Hartree atomic units.
C
C      ICT(NNE)  :
C      The number of asymptotically open channels at each energy.
C
C      RSYM(NE)  :
C      Used to test the symmetry of the R4MT matrix at each energy.
C
C      CENT(ID)  :
C      Array of angular momentum contribution to the D matrix.
C
C      DMI(ID)  :
C      Array containing diagonal elements of D matrix in the current
C      sector.
C
C***********************************************************************
C            *****Some arrays used by subroutines *****
C            *****     called by this program     *****
C            *****  (but not by program itself)   *****
C            *****     in alphabetical order      *****
C            ******************************************
C
C      AEMT(ID)-Used in GNSCAT.
C
C      AFMT(ID)-Used in GNSCAT.
C
C      AGMT(ID)-Used in GNSCAT.
C
C      AHMT(ID)-Used in GNSCAT.
C
C      AKMT(ID)-Used in EXSCAT, GNSCAT.
C
C      EFASY(ID,ID)-Used in GNSCAT, (POT).
C
C      ETA(ID)-Used in EXSCAT.
C
C      HPOS(NSECT)-Used in RPROP (is the same as H(NSECT) in RMPROP).
C
C      ISI(ID,2)-Used in GNSCAT, MATCH, SUMS.
C
C      ISJ(ID,NVIB)-Used in GNSCAT, MATCH (CALLED ISI IN THESE), SUMS,
C      (POT).
C
C      ISJ2(ID,NVIB)-Used in GNSCAT, SUMS.
C
C      JIN(ID,NAN)-Used in,  SUMS, EXSCAT, GNSCAT,
C      MATCH, EINT, (POT).
C
C      LAGS(16)-Used in GNSCAT.
C
C      P1(ID)-Used in RPROP.
C
C      P3INV(ID)-Used in RPROP.
C
C      R1MT(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R1MTM1(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R2MT(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R2MTM1(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R3MT(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R3MTM1(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R4MT(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      R4MTM1(ID,ID,NNE)-Used in GNSCAT, RPROP, DROP.
C
C      REACT(ID,ID)-Used in GNSCAT.
C
C      RSCAT(ID,ID)-Used in EXSCAT, GNSCAT.
C
C      SIM(ID,ID)-Used in EXSCAT, GNSCAT.
C
C      SR1(ID,ID)-Used in RPROP.
C
C      SR2(ID,ID)-Used in RPROP.
C
C      SR3(ID,ID)-Used in RPROP.
C
C      SR4(ID,ID)-Used in RPROP.
C
C      SRE(ID,ID)-Used in EXSCAT, GNSCAT.
C
C      TMPTAU(ID)-Used in TAUMTS.
C
C      U(ID,2)-Used in GNSCAT, SUMS.
C
C***********************************************************************
C              ***** Dimensions of arrays used*****
C              *****by this program and by the*****
C              *****   indicated subroutines  *****
C
C      DPAST(ID)-Used in RMPROP.
C
C      EFCNI(ID,ID)-Used in RMPROP, GNSCAT, MATCH, TAUMTS.
C
C      EFCNM1-Used in RMPROP, TAUMTS.
C
C      EI(ID)-Used in RMPROP, (POT), (EIGEN).
C
C      EVALI(ID)-Used in RMPROP, GNSCAT, MATCH, RPROP.
C
C      ICENT1(ID)-Used in RMPROP.
C
C      ICENT2(ID)-Used in RMPROP.
C
C      L(ID)-Used in RMPROP, GNSCAT, MATCH, (POT).
C
C      LBGS(15)-Used in RMPROP, GNSCAT, (PRSCAT).
C
C      LAM0SQ(ID)-Used in RMPROP (CALLED LMB0SQ), GNSCAT.
C
C      LAMSQ(ID,NNE)-Used in RMPROP, GNSCAT.
C
C      LGS(15)-Used in EXSCAT, GNSCAT, TITLES, (PRSCAT).
C
C      OAM(20)-Used in RMPROP, GNSCAT.
C
C      UPCDMT(ID,ID)-Used in RMPROP, (POT).
C
C      PMAT(ID,ID)-Used in RMPROP, EXSCAT, GNSCAT.
C
C      RABC(NSECT)-Used in RMPROP, EXSCAT, DROP, GNSCAT, TITLES.
C
C      SPRRE(ID,ID)-Used in RMPROP, GNSCAT.
C
C      SPRIM(ID,ID)-Used in RMPROP, GNSCAT.
C
C      SR1(ID,ID)-Used in RPROP.
C
C      SR2(ID,ID)-Used in RMPROP, RPROP, DROP.
C
C      TAUE(ID,ID,NSAVE)-Used in RMPROP, TAUMTS, TDUMP, TLOAD.
C
C      TAUI(ID,ID)-Used in RMPROP, RPROP, TAUMTS.
C
C      TEMPA(ID)-Used in RMPROP, GNSCAT.
C
C      TIMC(50) OR TIMEC(50)-Used in RMPROP, RPROP, GNSCAT.
C
C      TINVE(ID,ID,NSAVE)-Used in RMPROP, TAUMTS, TDUMP, TLOAD.
C
C      TINVI(ID,ID)-Used in RMPROP, RPROP, TAUMTS.
C
C***********************************************************************
C                          *****Namelist*****
C             *****Describes a number of the variables*****
C             *****read in to the program and some of *****
C             ***** the parameters and constants used *****
C             *****          by the program           *****
C             *****      (in alphabetical order)      *****
C             *********************************************
C
C        EPSDR-When the absolute values of all the off diagonal elements
C      of the matrix R4MT for a closed channel are less than EPSDR
C      that channel is effectively uncoupled from the other channels
C      and is neglected in the asymptotic analysis.
C        EPSRED-When the absolute values of all the elements of the
C      matrix SR2 for a given closed channel are less than EPSRED this
C      channel is dropped and the number of channels propagated is
C      reduced.  Typically LR4MT will be true when these two options are
C      used.
C        H-Enter here the first two stepsizes in atomic units.
C        HMAX-Maximum stepsize allowed in atomic units.
C        HMIN-Minimum stepsize allowed in atomic units
C        ID-Obviously the minimum required dimension of many of the
C      arrays used in this program is NDIM.  Thus you can use NDIM as
C      the defining dimension or you can over dimension the arrays.
C      Whatever value you chose for the DIMENSION statement for
C      arrays L and JIN (for example) is called ID and can be greateR
C      than or equal to NDIM and must be specified here.
C        JTOT-Integer value of the total angular momentum.
C        L-Integer array of the orbital angular momentum in each channel
C      (NDIM values).
C        LBES-If true match to Bessel functions and if false match to
C      sin/cos functions in GNSCAT.
C        LOPUT-If true print information on form of output.
C        LR4MT-If true only R4MT is propagated.  This should be true
C      only if the effective potential is very large in all channels at
C      the origin.
C        MUABC-Reduced mass in atomic units.
C        NAN-NAN is a dimension used in the arrays JIN, JIN2, JT among
C      others and is set to this value so that JIN can more easily store
C      intermediate values used in the calculation of angular integrals.
C        NDIM-Number of terms in the primitive basis set (sometimes this
C      is called ND).
C        NNE-The maximum number of multiple energies/angular momenta
C      which can be performed simultaneously ABS(NNE) should be less
C      than or equal to twenty (20).
C        NPROP--If = 1 The transpose of matrix TAU is used instead of
C      the inverse.  This is not rigorously justified for PDIM.NE.NDIM.
C        This is incompatible with the fast propagator, and the sector
C        R matrices will be calculated explicitly.
C               If = 0 or 2 use the inverse of matrix TAU :
C        If = 0 then calculate sector R matrices explicitly.
C        If = 2 then only have them implicitly, which is faster.
C        NSAVE-The maximum number of sectors in memory at one time.
C        NSECT-The dimension of arrays which are saved in each sector,
C      e.g. RABC, must be greater than or equal to nsect.  NSECT is
C      used to define the maximum number of sectors in which calcula-
C      tions will be carried out.  The program attemps to calculate
C      scattering information from the NSECT-3 sector onward even if
C      the asymptotic region has not yet been reached, and the program
C      terminates in the NSECT-1 sector even if the calculations are
C      not converged.
C        NVIB1-Used in dimensioning the interaction potential matrix.
C        RABC-Enter here the center of the first sector in atomic units.
C        RASYE-Some large value of RABC that is definitely in the
C      asymptotic region.
C        RBIG-The minimum value of rabc in atomic units for which the
C      program should begin to test whether the asymptotic regioN
C      has been reached.
C        RMAX-Maximum RASY allowed.  Subroutine GNSCAT sets LCONV
C      true for RASY.GT.RMAX.
C
C      Logical array LGS:       Print out:
C  LGS(1)   Each time the scattering matrix is printed also print R, the
C           midpoint of the sector, and EFACT.
C  LGS(2)   Print the scattering matrix.
C
C       Select one of the following options (j) by setting the integer
C       input variable IOPT2 equal to j.
C       Option (j)
C         (1)   Each time it is calculated.
C         (2)   For the previous and current sectors after the
C               calculations converge.
C         (3)   In the last sector only.
C
C  LGS(3)   Check unitarity and print checks each time the scattering
C           matrix is printed.
C
C  LGS(4)   In the last sector calculate and print the phase shifts.
C
C  LGS(5)   In the last sector calculate and print the cross sections.
C
C  LGS(6)   In the last sector calculate and print the transition
C           probabilities.
C
C  LGS(7)   In the last sector print the reactance matrix.
C
C  LGS(8)   In the last sector try to calculate and print the quantities
C           requested in LGS(4),LGS(5), and LGS(6) even if the
C           calculations are terminated before convergence is reached.
C
C  LGS(9)   Calculate the symmetrized reactance matrix.
C       Select one of the following options by setting
C       the integer input variable IOPT3 equal to j.
C       Option (j)
C         (1)   Symmetrize the reactance matrix by imposing the
C               lower left triangle on the upper right.
C         (2)   Symmetrize the reactance matrix by imposing the
C               upper right triangle on the lower left.
C         (3)   Symmetrize the reactance matrix by setting
C               RSCAT(I,J)=RSCAT(J,I) equal to the arithmetic
C               mean of the originally calculated off diagonal
C               elements.
C         (4)   Do all of the above.
C
C       LGS(9) must be true if any of the following are true:
C
C  LGS(10)  In the last sector print the symmetrized reactance matrix.
C
C  LGS(11)  In the last sector print the symmetrized scattering matrix
C           and check unitarity.
C
C  LGS(12)  From the symmetrized matrices calculate and print the
C           quantities requested in LGS(4), LGS(5), and LGS(6).
C
C  LGS(13)  Write the symmetrized reactance matrix onto tape 7 in
C           format requested by BRANDT, TRUHLAR, SMITH D.C.S. program.
C
C  LGS(14)   Write JTOT, the energy and the T matrix to tape 10 using
C            1X,I5,1PE15.8 AND 1X,1PE13.6 format. EXSCAT will do this.
C
C  LGS(15)   Calculate and write the eigenphases.  The eigenphases as a
C            function of R will be written with R on tape 11, and the
C            eigenphases, R and the eigenvectors will be written on tape
C            16.
C***********************************************************************
C**                  *****System subroutines*****
C**   From the Cray SCILIB scientific subprogram library subprograms
C**
C**     From the EISPACK set of subroutines:
C**
C**     SUBROUTINE RS   
C**
C**     FROM THE LAPACK SET OF SUBROUTINES:
C**
C**     FUNCTION DASUM, SUBROUTINE DAXPY, FUNCTION DDOT, SUBROUTINE DGEMM,
C**
C**     SUBROUTINE DGEMV, SUBROUTINE DGER, SUBROUTINE DGETF2,
C**
C**     SUBROUTINE DGETRF, SUBROUTINE DGETRS, SUBROUTINE DGETRI,
C**
C**     SUBROUTINE DLASWP, FUNCTION DNRM2, SUBROUTINE DSCAL,
C**
C**     SUBROUTINE DSWAP, SUBROUTINE DTRMM, SUBROUTINE DTRMV,
C**
C**     SUBROUTINE DTRSM, SUBROUTINE DTRTI2, SUBROUTINE DTRTRI,
C**
C**     FUNCTION IDAMAX, SUBROUTINE ILAENV, FUNCTION LSAME
C***********************************************************************
C***********************************************************************
      PROGRAM RMPROP
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION MUABC,LAM0SQ,LAMSQ,LAMSQE,LMB0SQ
      INTEGER PDIM,PD,OAM,NPROP
      INTEGER FU1,FU2,FU3,FU4,FU5,FU6,FU7,FU8,FU9,FU10,FU11,FU12,FU13,
     1        FU14,FU15,FU16,FU17,FU18,FU19

C***Include a file containing sizes of arrays.
C***Note that the INCLUDE statement is non-ANSI-standard FORTRAN.
C***Note that if NVIBX=1 then subroutine SUMS (LGS(3)) is inoperative.

      PARAMETER(FU1=1,FU2=2,FU3=3,FU4=4,FU5=5,FU6=6,FU7=7,FU8=8,
     1          FU9=9,FU10=10,FU11=11,FU12=12,FU13=13,FU14=14,
     2          FU15=15,FU16=16,FU17=17,FU18=18,FU19=19)
      INCLUDE 'parameter.inc'
      CHARACTER TITLE1*80,TIMDAT*80,TIMDAO*80,TITLE2*80
      COMMON/PTIMCM/PARTIM(10)
      COMMON/LCM/LEFACT,LBES,LCONV,LEARLY,LENTER,LR4MT,LSTOP1,LSTOP2
      COMMON/TIMCOM/TIMC(50)
      COMMON/PRNTCM/IUNITS,IFTXX
      COMMON/LINCM/ISCR(8*IDX)
      DIMENSION EI(IDX),L(IDX),DMI(IDX),LGS(15),H(NSECTX),ICENT(IDX),
     1          CENT(IDX),OAM(20),RSTART(20),EVALI(IDX),LAM0SQ(IDX),
     2          LPSYM(8),LAMSQ(IDX,NNEX),LAMSQE(IDX),RABC(NSECTX),
     3          NRD(NSECTX),JIN(IDX,NANX),PD(10),P1(IDX),
     4          P3INV(IDX),VCENT(IDX),E(20),IND(10,2)
      DIMENSION LMB0SQ(IDX,NNEX),EEV(NNEX),EVADD(NNEX),ICT(NNEX),
     1          NEWRED(NNEX),NRED(NNEX),NOPEN(NNEX),AKMT(IDX),AFMT(IDX),
     2          RSYM(NNEX),AGMT(IDX),AEMT(IDX),AHMT(IDX),EPSA(5),
     3          EPSB(5),EPSC(5),REPSA(5),REPSB(5),LPR(10),DMIM1(IDX),
     4          RCUSP(4),ETA(IDX),ICENT1(IDX),ICENT2(10),
     5          RPSM(20),U(IDX,2),ISJ(IDX,NVIBX),ISJ2(IDX,NVIBX),
     6          HMINA(5),HMINB(5),HMINC(5),HMAXA(5),HMAXB(5),HMAXC(5),
     7          LBGS(15),EVALE(IDX,NSECTX),AK0MT(IDX),TEMP4(IDX,8)
       INTEGER IPIV(IDX)

C***Big arrays:
C***R4MT, EFCNM1, and BPOT must exist at all times.
C***EFASY must exist at all times if LTASY is true.
C***SPRRE and SPRIM must exist at all times if convergence wrt
C***distance is controlled by EPSMAG and EPSPH, I.E. when LPSMH is true.
C***If NPROP = 1 OR 2, TINVE can be the same as TAUE.
C***TEMP3 can be TEMP2 if no more then one reactance matrix.
C***Symmetrization option is used in GNSCAT.
C***If NPROP = 2, and TYPE2 second energies are used, TEMP2 can
C***be the same as TAUE.

      COMMON/TEMPCM/TEMPAR(IDX,IDX)
      DIMENSION EFCNM1(IDX,IDX),BPOT(IDX,IDX,NVIB1X),
     1          EFASY(IDX,IDX),TEMPBC(IDX,IDX*2),TEMP2(IDX,IDX),
     2          TAUE(IDX,IDX,NSAVEX),TINVE(IDX,IDX,NSAVEX),
     3          SPRRE(IDX,IDX),SPRIM(IDX,IDX),TEMP3(IDX,IDX),
     4          R1MT(IDX,IDX,NNEX),R1MTM1(IDX,IDX,NNEX),
     5          R2MT(IDX,IDX,NNEX),R2MTM1(IDX,IDX,NNEX),
     6          R3MT(IDX,IDX,NNEX),R3MTM1(IDX,IDX,NNEX),
     7          R4MT(IDX,IDX,NNEX),R4MTM1(IDX,IDX,NNEX),
     8          TEMPR1(IDX,IDX),PCDMT(NBARX)
      LOGICAL   LASY,LBES,LCONV,LEARLY,LENTER,LR4MT,LSTOP1,LSTOP2,
     1          LTASY,LTHREW,LDUM,LOPUT,LPR,LPR1,LPR2,LRS,LSTAP,
     2          LTESTL,LEREF,LGS,LGS1,LCENT,LDWTT,LPSMH,
     3          LBGS,LWSTT,LRSFT,LEFACT,LTD,LPD,LLAST,LTYPE2,
     4          LPSINF,LPEPS,LTID,LPSYM,LNOINV,LJRI

      DATA LTHREW/.FALSE./
      DATA TIMDAT(1:20)/'                    '/
      DATA TIMDAT(21:40)/'                    '/
      DATA TIMDAT(41:60)/'                    '/
      DATA TIMDAT(61:80)/'                    '/


      OPEN(UNIT=FU5,FILE='fort.5',STATUS='OLD',FORM='FORMATTED')
      OPEN(UNIT=FU15,FILE='fort.15',STATUS='UNKNOWN',FORM='FORMATTED')

      CRIRS=-100.0

C***The next line initializes R4MT before reading it in.  If it is true
C***this means you must start propagation within classically forbidden
C***region (RSTART should be within the centrifugal barrier).

      LR4MT=.TRUE.

C***The next line sets LSTAP to .TRUE. so that by default RMROP stops   5M13R94
C***after completing the propagation for one potential.                 5M13R94
      LSTAP=.TRUE.                                                      5M13R94
C***TIMC(1) is a timing of reading the information from FORTRAN unit 5.

      CALL SECOND(TIM1)
      TIMC(1)=TIM1

C***Read in various parameters from FORTRAN unit 5.
C***The first output written to FORTRAN unit 15 should always be a
C***1-line description of the run (i.e. potential, basis size, energy)
C***read from UNIT 5 here and write this to FORTRAN units 6 and 15.

      READ(5,1000)TITLE1
      READ(5,1000)TITLE2

C***Title for program.
C***The first output written to user-readable FORTRAN units should be a
C***title for the version of the program and the date and time.

      WRITE(6,1010)
      WRITE(15,1010)
      WRITE(6,1015)
      WRITE(15,1015)

      WRITE(6,1020)
      CALL DATTIM(6,TIMDAT)
      WRITE(15,1022)TIMDAT(1:80)
    
C***1-line description of the run (i.e. potential, basis size, energy)
C***and a 1-line description of its purpose (test suite, convergence).
C***We write this to FORTRAN units 6 and 15.

      WRITE(6,1000)TITLE1
      WRITE(15,1000)TITLE1
      WRITE(6,1000)TITLE2
      WRITE(15,1000)TITLE2

C***The variable LJRI is a logical variable 'LOGICAL JUST READ INPUT'
C***If it is true then just read and echo input from unit 5 and stop.
C***If it is false continue with the entire program.

      READ(5,1040)LJRI,LR4MT
      IF(LJRI)WRITE(6,1050)
      IF(LR4MT)WRITE(6,1060)
      IF(.NOT.LR4MT)WRITE(6,1070)
      READ(5,1080)ID,NSECT,NAN,NSAVE,NVIB,NVIB1,NNE
      IF(LJRI) WRITE(6,1090)ID,NSECT,NAN,NSAVE,NVIB,NVIB1,NNE
      READ(5,1100) IREST,ISAVE,ISAVV,IUNXX,IFTXX
      IF(LJRI) WRITE(6,1110)IREST,ISAVE,ISAVV,IUNXX,IFTXX

C***Write addition to HEADER if this is a restart run.

      IF(IREST.NE.0.AND.(.NOT.LJRI))THEN
      OPEN(UNIT=FU3,FILE='fort.3',STATUS='OLD',FORM='FORMATTED')
      READ(3,1022)TIMDAO(1:80)
      CLOSE(UNIT=FU3)
      WRITE(6,1130)TIMDAO(1:80)
 1130 FORMAT(//1X,45HThis is a restart run.  It continues the run ,
     111Hstarted at ,/,A80/)
      END IF

C***Initialize values to be read in to avoid compilation error messages.
C***EPSMAG is an epsilon for convergence of the magnitude of
C***user-selected scattering matrix elements with respect to RMAX.
C***Here we initialize it to zero so if we never read it in we skip
C***the convergence test.

      EPSMAG=0D0

C***EPSPH is an epsilon for convergence of the phase of user-selected
C***scattering matrix elements with respect TO RMAX.
C***Here we initialize it to zero so if we never read it in we skip
C***the convergence test.

      EPSPH=0D0

C***NIND is the number of scattering matrix elements for which to test
C***the convergence using EPSMAG and EPSPH if desired.

      NIND=0

C***IOPT2 and IOPT3 are used in the asymptotic analysis.

      IOPT2=1
      IOPT3=1

C***Begin reading data for run.

      READ(5,1140) MUABC,NDIM,JTOT
      IF(LJRI) WRITE(6,1150)MUABC,NDIM,JTOT

C***Stop run if NDIM read in exceeds array dimensions set by IDX.

      IF(NDIM.GT.IDX)WRITE(6,1160)NDIM,IDX
 1160 FORMAT(/1X,24HThe input value of NDIM ,I4,13H exceeds the ,
     114Hparameter IDX ,1X,I4,1H.)
      IF(NDIM.GT.ID)STOP 5100

C***Continue reading in parameters.

      READ(5,1100) (L(I),I=1,NDIM)
      IF(LJRI) WRITE(6,1170)(L(I),I=1,NDIM)
      READ(5,1180) NPD,NE,IROWS
      IF(LJRI) WRITE(6,1190)NPD,NE,IROWS

C***The next line is a trap involving multiple energy runs.
C***NE determines the # of multiple energy runs and NNE is the maximum
C***allowed by the code.

      IF(ABS(NE).GT.NNE)STOP 5200

C***Continue reading in parameters.

      READ(5,1100) (PD(I),I=1,NPD)
      IF(LJRI) WRITE(6,1200)(PD(I),I=1,NPD)
      READ(5,1210) (E(I),RSTART(I),OAM(I),I=1,ABS(NE))
      IF(LJRI) WRITE(6,1220)(E(I),RSTART(I),OAM(I),I=1,ABS(NE))
      READ(5,1230) H(1),H(2),RMAX
      IF(LJRI)WRITE(6,1240)H(1),H(2),RMAX
      READ(5,1250) NEPS
      IF(LJRI)WRITE(6,1260)NEPS
      READ(5,1270) (EPSA(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1280)(EPSA(I),I=1,NEPS)
      READ(5,1270) (EPSB(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1290)(EPSB(I),I=1,NEPS)
      READ(5,1270) (EPSC(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1300)(EPSC(I),I=1,NEPS)
      READ(5,1270) (HMINA(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1310)(HMINA(I),I=1,NEPS)
      READ(5,1270) (HMINB(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1320)(HMINB(I),I=1,NEPS)
      READ(5,1270) (HMINC(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1330)(HMINC(I),I=1,NEPS)
      READ(5,1270) (HMAXA(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1340)(HMAXA(I),I=1,NEPS)
      READ(5,1270) (HMAXB(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1350)(HMAXB(I),I=1,NEPS)
      READ(5,1270) (HMAXC(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1360)(HMAXC(I),I=1,NEPS)
      READ(5,1270) (REPSA(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1370)(REPSA(I),I=1,NEPS)
      READ(5,1270) (REPSB(I),I=1,NEPS)
      IF(LJRI)WRITE(6,1380)(REPSB(I),I=1,NEPS)
      READ(5,1250) NCUSP
      IF(LJRI)WRITE(6,1390)NCUSP
      READ(5,1270) (RCUSP(I),I=1,NCUSP)
      IF(LJRI)WRITE(6,1400)(RCUSP(I),I=1,MAX(1,NCUSP))
      READ(5,1410) EPSRED,RBIG,NPROP
      IF(LJRI)WRITE(6,1420)EPSRED,RBIG,NPROP
      READ(5,1430) LPSINF,LPSYM,LOPUT
      IF(LJRI)WRITE(6,1440)LPSINF,LPSYM,LOPUT
      READ(5,1430) LPR
      IF(LJRI)WRITE(6,1450)LPR
      READ(5,1460) EPSDR,RASYE
      IF(LJRI)WRITE(6,1470)EPSDR,RASYE
      READ(5,1480) LBES,LTASY,LPEPS,IMATCH,ITPRNT,ITCOL
      IF(LJRI)WRITE(6,1490)LBES,LTASY,LPEPS,IMATCH,ITPRNT,ITCOL
      READ(5,1250) IPMX
      IF(LJRI)WRITE(6,1500)IPMX
      READ(5,1270) (RPSM(I),I=1,IPMX)
      IF(LJRI)WRITE(6,1510)(RPSM(I),I=1,IPMX)
      IF(LJRI)CALL PRSCAT(ID,LGS,IOPT2,EPSMAG,EPSPH,LGS1,IOPT3,LBGS,IND,
     1NIND)
      IF(LJRI)WRITE(6,1520)(LGS(I),I=1,15)
      IF(LJRI)WRITE(6,1530)(LBGS(I),I=1,15)
      IF(LJRI)WRITE(6,1540)IOPT2,IOPT3
      IF(LJRI)WRITE(6,1550)EPSMAG,EPSPH
      IF(LJRI.AND.(EPSMAG.NE.0.0.AND.EPSPH.NE.0.0))WRITE(6,1560)NIND
      IF(LJRI.AND.(EPSMAG.NE.0.0.AND.EPSPH.NE.0.0))WRITE(6,1570)
     1(IND(I,1),I=1,MIN0(NIND,10))
      IF(LJRI.AND.(EPSMAG.NE.0.0.AND.EPSPH.NE.0.0))WRITE(6,1580)
     1(IND(I,2),I=1,MIN0(NIND,10))

 1040 FORMAT(2L1)
 1050 FORMAT(/1X,50HThis run will only read and echo data from FORTRAN,
     17H unit 5,1H./)
 1060 FORMAT(/1X,44HLR4MT is true so use only the R4MT submatrix,1H./)
 1070 FORMAT(/1X,48HLR4MT is false so use all four submatrices R1MT ,
     114HR2MT R3MT R4MT,1H./)
 1080 FORMAT(7I5)
 1030 FORMAT(1X,24HThis run is starting at ,A8,4H ON ,A8,1H./)
 1000 FORMAT(A72)
 1100 FORMAT(10I5)
 1140 FORMAT(G15.6,2I5)
 1180 FORMAT(3I5)
 1210 FORMAT(2G15.6,I5)
 1230 FORMAT(3G15.6)
 1250 FORMAT(I5)
 1270 FORMAT(5G15.6)
 1410 FORMAT(2G15.6,I5)
 1430 FORMAT(10L1)
 1460 FORMAT(2G15.6)
 1480 FORMAT(3L1,3I5)
 1090 FORMAT(43HRecord 4: ID,NAN,NSECT,NSAVE,NVIB,NVIB1,NNE,/7I5)
 1110 FORMAT(39HRecord 5: IREST,ISAVE,ISAVV,IUNXX,IFTXX,/10I5)
 1150 FORMAT(25HRecord 6: MUABC,NDIM,JTOT,/G15.6,2I5)
 1170 FORMAT(23HRecord 7: L(I),I=1,NDIM,/10I5)
 1190 FORMAT(22HRecord 8: NPD,NE,NROWS,/3I5)
 1200 FORMAT(23HRecord 9: PD(I),I=1,NPD,/10I5)
 1220 FORMAT(39HRecord 10: E(I),RSTART(I),OAM(I),I=1,NE,/2G15.6,I5)
 1240 FORMAT(25HRecord 11: H(1),H(2),RMAX,/3G15.6)
 1260 FORMAT(15HRecord 12: NEPS,/I5)
 1280 FORMAT(27HRecord 13: EPSA(I),I=1,NEPS,/5G15.6)
 1290 FORMAT(27HRecord 14: EPSB(I),I=1,NEPS,/5G15.6)
 1300 FORMAT(27HRecord 15: EPSC(I),I=1,NEPS,/5G15.6)
 1310 FORMAT(28HRecord 16: HMINA(I),I=1,NEPS,/5G15.6)
 1320 FORMAT(28HRecord 17: HMINB(I),I=1,NEPS,/5G15.6)
 1330 FORMAT(28HRecord 18: HMINC(I),I=1,NEPS,/5G15.6)
 1340 FORMAT(28HRecord 19: HMAXA(I),I=1,NEPS,/5G15.6)
 1350 FORMAT(28HRecord 20: HMAXB(I),I=1,NEPS,/5G15.6)
 1360 FORMAT(28HRecord 21: HMAXC(I),I=1,NEPS,/5G15.6)
 1370 FORMAT(28HRecord 22: REPSA(I),I=1,NEPS,/5G15.6)
 1380 FORMAT(28HRecord 23: REPSB(I),I=1,NEPS,/5G15.6)
 1390 FORMAT(16HRecord 24: NCUSP,/I5)
 1400 FORMAT(29HRecord 25: RCUSP(I),I=1,NCUSP,/5G15.6)
 1420 FORMAT(28HRecord 26: EPSRED,RBIG,NPROP,/2G15.6,I5)
 1440 FORMAT(29HRecord 27: LPSINF,LPSYM,LOPUT,/10L1)
 1450 FORMAT(14HRecord 28: LPR,/10L1)
 1470 FORMAT(22HRecord 29: EPSDR,RASYE,/2G15.6)
 1490 FORMAT(47HRecord 30: LBES,LTASY,LPEPS,IMATCH,ITPRNT,ITCOL
     1,/3L1,3I5)
 1500 FORMAT(15HRecord 31: IPMX,/I5)
 1510 FORMAT(27HRecord 32: RPSM(I),I=1,IPMX,/5G15.6)
 1520 FORMAT(24HRecord 33: LGS(I),I=1,15,/20L1)
 1530 FORMAT(25HRecord 34: LBGS(I),I=1,15,/20L1)
 1540 FORMAT(22HRecord 35: IOPT2,IOPT3,/2I4)
 1550 FORMAT(23HRecord 36: EPSMAG,EPSPH,/2G16.5)
 1560 FORMAT(15HRecord 37: NIND,/2I4)
 1570 FORMAT(37HRecord 38: IND(I,1),I=1,MIN0(NIND,10),/10I4)
 1580 FORMAT(37HRecord 39: IND(I,2),I=1,MIN0(NIND,10),/10I4)

C***Stop here if we just want to read and echo input.

      IF(LJRI)STOP 5099

C***Write out miscellaneous information concerning the output.

      WRITE(6,1590)
      WRITE(6,1600)(L(I),I=1,NDIM)
      WRITE(6,1610)JTOT
      WRITE(6,1620)MUABC
      WRITE(6,1630)ID,NSAVE,NSECT,RBIG,H(1),H(2)
      IF(LR4MT)WRITE(6,1640)
      IF(NPROP.EQ.2.AND.LR4MT)THEN
      WRITE(6,1650)
 1650 FORMAT(/1X,'The fast R matrix propagation formulas will be used.')
      END IF
      IF(NPROP.EQ.2.AND..NOT.LR4MT)THEN
      WRITE(6,1660)
 1660 FORMAT(/1X,51HYou have selected NPROP=2.  This choice selects the,
     1/1X,57H fast R matrix propagation formulas.  But this choice is ,
     2/1X,56H inconsistent with LR4MT=.FALSE..  NPROP is set to zero.)

      NPROP=0
      END IF
      LNOINV=.FALSE.
      IF(NPROP.EQ.1)LNOINV=.TRUE.
      IF(NPROP.EQ.1)WRITE(6,1670)
      IF(NPROP.NE.1)WRITE(6,1680)

 1010 FORMAT(/,1X,'RMPROP R matrix propagation code version 2.1.2')
 1015 FORMAT(1X,'Version Date: December 21, 1994',/)
 1020 FORMAT(18X,1H*,18X,1H*,18X,1H+,16X,1H*,
     2/,1X,'Melissa S. Reeves, Michael J. Unekis, David W. Schwenke'
     & ,', Nancy M. Harvey,',/,24x,'*',/,4x,
     3 'and Donald G. Truhlar',/,1X,24HDepartment of Chemistry ,
     455H, Chemical Physics Program, and Supercomputer Institute,
     5/,1X,46HUniversity of Minnesota, Minneapolis, MN 55455,
     6/,1X,54Hand NASA Ames Research Center, Moffet Field, CA, 94035,
     7/,1H*   ,/1X,24HUniversity of Minnesota ,/,1H+,/,1X,9HNASA Ames,
     816H Research Center//)
 1022 FORMAT(A80)
 1590 FORMAT(1X,41HOrbital angular momentum in each channel:/)
 1600 FORMAT(1X,17I4)
 1610 FORMAT(/1X,22HTotal angular momentum,I5,1H./)
 1620 FORMAT(1X,12HReduced mass,1X,1PE12.6/)
 1630 FORMAT(1X,26HMaximum number of channels,I8,1H.//1X,
     139HNumber of sectors in memory at one time,I8,1H.//1X,
     225HMaximum number of sectors,I8,1H.,10X,7HRBIG = ,1PE13.7//1X,
     328HStepsize in first sector =  ,E13.7,1H./,1X,
     428HStepsize in second sector = ,E13.7,1H./)
1640  FORMAT(1X,42HOnly R4MT will be propagated therefore the,
     135H effective potential should be very/,
     237H large in all channels at the origin.)
1670  FORMAT(/1X,35HIn the sector R matrix calculations,
     143H the transpose of the overlap matrix TAU is/1X,73H used instead
     2 of the inverse.  The scattering matrix is thus forced to be,/,
     39H unitary.)
1680  FORMAT(/1X,35HIn the sector R matrix calculations,
     141H the inverse of the overlap matrix TAU is/,1X,10Hused.  The,
     260H unitarity of the final scattering matrix is a good check of/,
     344H whether the overlap matrix O is orthogonal.)

C***Write information about matching the wave functions in the
C***asymptotic region.

      IF(LBES)WRITE(6,1690)
      IF(.NOT.LBES)WRITE(6,1700)
      WRITE(6,1710)
      IF(IMATCH.EQ.0)WRITE(6,1720)
      IF(IMATCH.EQ.1)WRITE(6,1730)
      IF(IMATCH.EQ.2)WRITE(6,1740)

 1690 FORMAT(/1X,51HIn the asymptotic region wave functions are matched,
     1/1X,28Hto Ricatti-Bessel functions.)
 1700 FORMAT(/1X,51HIn the asymptotic region wave functions are matched,
     1/1X,21Hto sin-cos functions.)
 1710 FORMAT(/1X,48HIn the asymptotic region, the matching functions,
     113H will use the)
 1720 FORMAT(1X,19Hasymptotic momenta./)
 1730 FORMAT(1X,53Hlocal momenta calculated at the middle of the sector.
     1/)
 1740 FORMAT(1X,48Hlocal momenta calculated at the sector boundary./)

C***Write information about neglecting closed channels in the
C***asymptotic analysis.

      WRITE(6,1750)EPSDR
 1750 FORMAT(1X,38HThe criterion for neglecting uncoupled,
     11X,33Hclosed channels in the asymptotic,/
     21X,19Hanalysis is EPSDR = ,1PE10.2,1H.//1X)

C***Write information about the number and location of cusps in the
C***potential, if there are any.

      IF(NCUSP.NE.0)WRITE(6,1760)NCUSP,(RCUSP(I),I=1,NCUSP)
 1760 FORMAT(1X,39HThe number of cusps in the potential = ,I5/1X,
     130Hposition of cusps (in bohr) = ,5(1PE12.6,2X))

C***Write information about printing out the scattering matrix or other
C***results at intermediate distances.

      IF(IPMX.GT.0)WRITE(6,1770)(RPSM(I),I=1,IPMX)
 1770 FORMAT(/,1X,34HThe S matrix will be printed at R=,5(1PE11.3,1H,),/
     1,5(1PE11.3,1H,))

C***Set logical variables defining output printed by RMPROP.

      LPR1=LPR(5).OR.LPR(7).OR.LPR(8).OR.LPR(9)

C***Definitions derived from input variables and checks on consistency
C***of input.

      NEA=NE
      LTYPE2=NE.LT.-1.AND.IROWS.GE.0
      NE=ABS(NE)
      NEB=1
      IF(LTYPE2)NEB=NE
      ISECT=0
      KTDUMP=0
      IUNITS=19
      RSYMTL=1.0D-2

C***Set beginning center of mass separation in bohr for first energy
C***or basis.

      RABC(1)=RSTART(1)

C***INR is used to align starting separations for multiple energy runs.

      INR=0

C***LPSMH is used to govern printing out information at intermediate
C***distances.  Subroutine PRSCAT reads another array, called LBGS,
C***whose elements have the same meaning as the array LGS, except that
C***LBGS pertains to intermediate distances, as defined by the array
C***RPSM.  In the first sector where RABC(IS).GE.RPSM(I), the logical
C***variable LPSMH is set to true just before the call to GNSCAT.  In
C***GNSCAT, the array LGS is stored in LAGS, LBGS is stored in LBGS,
C***and the values of another array called LGS are determined by the
C***values in LPSMH.  If LPSMH is true LGS is set equal to LBGS, and if
C***LPSMH is false LGS is set equal to LAGS.  GNSCAT returns if
C***convergence has not been reached and RMAX has not been reached only
C***if LPSMH is false.

      LPSMH=.FALSE.

C***EVADD is the amount by which one adjusts the elements of the
C***D matrix for multiple energy runs. By definition it should be zero
C***for the first energy.

      EVADD(1)=0.D0

C***AMADD is like EVADD except it governs the centrifugal contribution
C***to the D matrix element.

      AMADD=0.D0

C***TIME1 is used to time the run.

      TIME1=0.0D0

C***INR1 is used to align starting distances of multiple runs.

      INR1=0

C***YITPST stores centrifugal contributions to the diagonal D matrix.

      YITPST=0.0D0

C***LTESTL is used to align starting distances of multiple runs.

      LTESTL=.FALSE.

C***LWSTT is 'LOGICAL WRITE STUFF TO TAPE' and governs writing to
C***FORTRAN unit FOR014 for second energy runs to be performed at a
C***later date.  LRSFT is 'LOGICAL READ STUFF FROM TAPE' and governs
C***reading RABC(IS) and H(IS) from FOR012.  These variables are
C***initialized here.  They are set to their values as used in the run
C***according to the input value of "IROWS".  Further information on
C***IROWS is given later in the program.

      LWSTT=.FALSE.
      LRSFT=.FALSE.

C***IPRINT governs which of the intermediate distances RPSM(I) is the
C***next one that intermediate information should be printed out at.

      IPRINT=1

C***LCENT is a logical variable determining whether the centrifugal
C***contribution to the potential is calculated from the L(I) or thE
C***OAM(I).

      LCENT=.FALSE.

      DO 4 I=1,NDIM
    4    IF(L(I).GT.0)LCENT=.TRUE.
      DO 3 I=1,NE
         IF((OAM(I).GT.0).AND.LCENT)GO TO 2
    3 CONTINUE
      GO TO 1
    2 WRITE(6,1780)
 1780 FORMAT(1X,37HEither all L or all OAM must be zero.)
      STOP 5300
    1 IF(.NOT.LCENT)WRITE(6,1790)
 1790 FORMAT(/1X,40HJTOT is set to the current value of OAM.)

C***NSECT3 and NSECT1 are used so that the program attempts to
C***calculate scattering information from the NSECT-3 sector onward
C***even if the asymptotic region has not been reached and stops in the
C***NSECT-1 sector.

      NSECT3=NSECT-3
      NSECT1=NSECT-1

C***Define factors involving the reduced mass used to change the
C***translational energy to the wave vector.

      TMU=2.0D0*MUABC
      TMU2=TMU*TMU
      YTMU=1.0D0/TMU

C***Define the left-hand-sides of the first and second sectors and
C***issue a flag and stop if the beginning of the first sector would
C***be .lt. zero.

      R0=RABC(1)-H(1)*0.5D0
      IF(R0.LT.0.0D0)STOP 5400
      RABC(2)=RABC(1)+0.5D0*(H(1)+H(2))

C***Set up a trap if the input parameters call for multiple stepsizes
C***and multiple energies simultaneously.

      IF(NEPS.GT.1.AND.ABS(NE).GT.1)STOP 5500

C***Set up a warning if the center of the first sector is farther out
C***than the first cusp (if any) in the potential.

      IF((NCUSP.GT.0).AND.(RCUSP(1).LE.RABC(1)))WRITE(6,1800)
 1800 FORMAT(///1X,48H*****Warning, the center of the first sector is ,
     122Hbeyond the first cusp.//)

C***LDWTT is 'LOGICAL DON'T WRITE TO TAPE' and is associated with the
C***variable "IROWS", used to decide whether to read/write from FORTRAN
C***units 8 and 14 for later date second energy runs.  Further
C***information on IROWS is given later in the program.

      LDWTT=.FALSE.

C***Set the maximum number of sectors allowed in a run.

      ISMAX=NSECT

C***ISAVV can be used for large runs.
C***One would do a run with ISAVV=-1 and ISMAX=100, and look at the
C***output written to UNIT 19.  If this is ok one can continue.
C***Usually one ignores this option--just set ISMAX impossibly large
C***and ISAVV=0, and use fort.17 and fort.18 for restart information.

      IF(ISAVV.LE.0)ISAVV=ISMAX

C***Set ISAVU and ISAVUP which are the unit numbers to write restart
C***information to alternately, i.e. fort.17 then fort.18 then fort.17
C***again, then fort.18 again, etc.  The restart information is written
C***every ISAVE sectors and is used for large runs so that they can
C***fit in a smaller queue--one submits a large run on a given queue
C***for execution, and when the cpu limit has expired one is left with
C***a restart file which can be read and the job continued from that
C***point.

      ISAVU=17
      ISAVUP=18

C***ILRS and LRS are associated with manual convergence with respect
C***to RSTOP and are no longer used.

      ILRS=ISMAX

C***The following line inhibits writing to tape unit14 if we are doing
C***multiple energies.  The line after it says that if we should write
C***RABC(IS) and H(IS) to tape unit 12 then we do it every NSAVE sectors.

      IF((NPD.EQ.1).AND.(NEA.LE.1))LDWTT=.TRUE.
      IF(.NOT.LDWTT.AND.ISAVE.NE.0)ISAVE=NSAVE

C***The next line tells us "where to go" for IROWS.lt.0., IROWS.eq.0,
C***or IROWS.gt.0--briefly if :
C
C      IROWS = -1 we read from units for008 and for014 to do
C              a second energy run where the first energy was done
C              at an earlier date (go to line 26).
C
C      IROWS = 1 we write information to units for008, for009 and
C                for014 to do do a restart run at a later date.
C
C      IROWS = 0 nothing happens, the code goes as normally (statement
C                47).
C
C      IROWS = 2 we write RABC(IS) and H(IS) to for012 (statement 48).
C
C      IROWS = 3 we read RABC(IS) and H(IS) from for012 (statement 48).
C
C      IROWS = 4 we read stepsizes like for IROWS=3 and
C
C                we write restart info like for IROWS=1 (statement 48).
C***

      IF(IROWS)26,47,48
  26  OPEN(UNIT=FU14,FILE='fort.14',STATUS='OLD',FORM='UNFORMATTED')
      READ(14)ISMAX
      INE=1
      READ(14)E(1),OAM(1),(RABC(I),H(I),NRD(I),(EVALE(J,I),J=1,NDIM),I=
     11,ISMAX),TIMDAO(1:80),(LAMSQE(I),I=1,NDIM),ILRS,((EFCNM1(I,J),
     2I=1,NDIM),J=1,NDIM)
      OPEN(UNIT=FU8,FILE='fort.8',STATUS='OLD',FORM='UNFORMATTED')
      WRITE(6,1810)E(1),TIMDAO
 1810 FORMAT(/,1X,28HThis is a second energy run./,1X,24HThe reference e
     1nergy was,1X,1PE15.7,/,1X,29HThe reference run was made on,1X,/
     2,A80/)
      EVADD(1)=TMU*(E(1)-E(2))
      AMADD=DBLE(OAM(2)*(OAM(2)+1)-OAM(1)*(OAM(1)+1))
      GO TO 47
   48 IF(IROWS.EQ.1)LDWTT=.FALSE.
      IF(IROWS.EQ.2)LWSTT=.TRUE.
      IF(IROWS.GE.3)LRSFT=.TRUE.
      IF(IROWS.EQ.4)LDWTT=.FALSE.
      IF(.NOT.LDWTT.AND.ISAVE.NE.0)ISAVE=NSAVE
      IF(.NOT.LDWTT)OPEN(UNIT=FU8,FILE='fort.8',STATUS='NEW',
     1FORM='UNFORMATTED')

C***Trap to identify when someone tries to use NPROP = 1 when dropping
C***channels which has not been shown to be rigorously allowed.

   47 DO 5 I=1,NPD
      IF((NDIM.NE.PD(I)).AND.(NPROP.EQ.1) )GO TO 9998
    5 CONTINUE

C***Go around the write statement triggered by the trap in the loop 5.

      GO TO 9997
 9998 WRITE(6,1820)
 1820 FORMAT(/////1X,43H*****Warning, you have chosen to substitute,
     154HTAU transpose for TAU inverse.  This is not justified ,
     217Hfor PDIM.NE.NDIM.//)
 9997 CONTINUE

C***Marker for starting reference calculations at energies other than
C***the first and for checking the existence of cusps in the potential.

      ICUSP=0
      IAFTER=0
      NPOT=0

C***All potential approximations.
C***Define approximate potential
C***and calculate the internal energy levels EI(I).

   21 CONTINUE
      NPOT=NPOT+1
      CALL SECOND(TIMET1)
      TIMC(1)=TIMET1-TIMC(1)
      CALL PREPOT(LSTAP,NDIM,JIN,ID,L,JTOT,BPOT,ISJ,EI,EFASY)

C***Initialize variables used for timing various parts of the code.
C***TIMEP is the time calculating the potential.

      TIMEP=0D0
      DO 52 IT=3,50
   52 TIMC(IT)=0.0
      TIMER=0D0
      TIMEG=0D0
      TIMET=0D0

C***TIMEDI is the time to diagonalize the interaction potential and to
C***construct the D matrix from the diagonalized potential matrix.

      TIMEDI=0D0
      TIMED=0D0
      CALL SECOND(TIMET2)
      TIMC(2)=TIMET2
      TIMC(2)=TIMC(2)-TIMET1

C***Set the centrifugal contribution to the energy (the orbital angular
C***momentum).

      DO 20 I=1,NDIM
      ICENT1(I)=L(I)*(L(I)+1)
      ICENT2(1)=OAM(1)*(OAM(1)+1)
      ICENT(I)=ICENT1(I)
      IF(.NOT.LCENT)ICENT(I)=ICENT2(1)
      AI=DBLE(ICENT(I))
   20 CENT(I)=AI*YTMU

C***Write out the internal energies.

      WRITE(6,1830)
 1830 FORMAT(/1X,22HInternal energy levels)
      WRITE(6,1840) (EI(I),I=1,NDIM)

C***Define desired scattering information.
C***Subroutine PRSCAT sets the values of the LGS to decide which
C***scattering output will be printed.

      CALL PRSCAT(ID,LGS,IOPT2,EPSMAG,EPSPH,LGS1,IOPT3,LBGS,IND,NIND)

C***Write description of how to read output printed for each sector.
C***If LOPUT is true print out information on the form of the output.
      IF(LOPUT)CALL TITLES(IDUM,1,DUM,ID,NSECT,LGS,RABC)

C***Loop over contracted basis set dimension.
C***Assign the initial number of channels for each basis set dimension.

      DO 225 IP=1,NPD
      PDIM=PD(IP)

C***The next two lines set LPD which tells whether or not we are doing
C***more than one propagated dimension.

      LPD=.FALSE.
      IF(IP.EQ.1)LPD=.TRUE.
      WRITE(6,1850)NDIM,PDIM,EPSRED
 1850 FORMAT(/33H Dimension of primitive basis set,I3/
     1,34H Dimension of contracted basis set,I3/
     2,35H The criterion to reduce the number,
     333H of channels propagated, EPSRED =,1PE10.2)

C***First energy calculations.
C***Define reference energy E(1).

      LEREF=.FALSE.
      IE=1

C***Recall if IROWS.EQ.-1 we will read in information for a second
C***energy run.

      IF(IROWS.EQ.-1)IE=2

C***EEV converts the first energy from hartree to electron volts.

      EEV(1)=E(IE)*27.21161

C***Define reference mode calculations.

      IF((IE.EQ.1).AND.(IP.EQ.1))LEREF=.TRUE.
      IF(.NOT.LEREF)GO TO 30

C***Addition for reference energy.

      NRED(1)=ID
      IF(.NOT.LPR(1))GO TO 25

C***Assign and write starting time of calculation.
C***Write reference energy in hartree and electron volts.

      CALL SECOND(TIME1)
      WRITE(6,1860)TIME1
   25 WRITE(6,1870)E(1),EEV(1)
 1860 FORMAT(/1X,25HStart calculation at time,2X,F9.3,2X,6Hin sec)
 1870 FORMAT(///1X,16HReference energy,1X,1PE15.8,1X,9Hhartree =,1X
     1,1PE15.8,1X,2HeV)

C***Initial things for TYPE2 second energy runs.

      IF(.NOT.LTYPE2)GO TO 401

C***Convert second energies from hartree to electron volts.
C***Check to see if starting center of mass distance is identical for
C***all energies, and define energy correction for second energies (will
C***be added to eigenvalues of potential energy matrix).

      DO 402 I=2,NE
      EEV(I)=27.21161*E(I)
      IF(RSTART(I).NE.RSTART(1))STOP 5600
      EVADD(I)=TMU*(E(I-1)-E(I))
  402 CONTINUE

C***Write out second energies in hartree then in electron volts.

      WRITE(6,1880)(E(I),I=2,NE)
      WRITE(6,1890)(EEV(I),I=2,NE)
 1880 FORMAT(1X,21HSecond energies in eh/(1X,8(1PE15.8)))
 1890 FORMAT(1X,21HSecond energies in eV/(1X,8(1PE15.8)))

C***If (.not.LTYPE2) we jump to this line.

  401 CONTINUE

C***Calculate quantities for the asymptotic boundary conditions.
C***Centrifugal potential should have vanished by RASYE.

      CALL SECOND(TIMEP1)

C***First calculate matrix elements of the potential at the asymptotic
C***separation.

      CALL POT(RASYE,NDIM,ID,BPOT,TEMPBC(1,1))
      CALL SECOND(TIMEP2)
      TIMEP=TIMEP+TIMEP2-TIMEP1

C***Calculate other contributions to the D matrix.  Note that YIT is
C***zero at long range.  Combine all into VCENT.

      YIT=0.0
      DO 388 I=1,NDIM
      VCENT(I)=CENT(I)*YIT+EI(I)-E(IE)
  388 CONTINUE

C***Combine the unpacked interaction matrix and the D matrix.

      VSUM=0D0
      DO 304 I=1,NDIM
      TEMPBC(I,I)=TEMPBC(I,I)+VCENT(I)
      DMI(I)=TEMPBC(I,I)
      VSUM=VSUM+ABS(TEMPBC(I,1))
  304 CONTINUE
      VSUM=VSUM-ABS(TEMPBC(1,1))

C***Diagonalize the unpacked interaction matrix at long range.  The
C***vector LAMSQE contains the eigenvalues and the array TEMPBC contains
C***the eigenvectors.
C***Either Cray SCILIB version of EISPACK RS or EV2RSP is called 
C***depending on the auxilliary files compiled.
C***Subroutine EV2RSP is an alternative routine based on Stephen Elbert's
C***EVVRSP.

      CALL EV2RSP(0,NDIM,NDIM,NBARX,IDX,PCDMT,TEMP4,ISCR,LAMSQE,        5M9R94
     1            TEMPBC,0,IERR)
      CALL RS(IDX,NDIM,TEMPBC,LAMSQE,1,TEMPBC,U,U(1,2),IERR)


C***Trap to catch error exit from asymptotic diagonalization.
C***Write out error message when problems occur.

      IF(IERR.NE.0)WRITE(6,1900)IERR
 1900 FORMAT(1X,7HIERR is,I10,27H for asymptotic calculation)
      IF(IERR.NE.0)STOP 5700

C***Multiply eigenvalues by TMU which is twice the reduced mass over
C***H-BAR (Planck's constant over 2*pi) in atomic units. Do it after
C***the diagonalization to make the arithmetic simpler.

      DO 389 I=1,NDIM
      LAMSQE(I)=TMU*LAMSQE(I)
  389 CONTINUE
      CALL SECOND(TIMDI2)
      TIMEDI=TIMEDI+TIMDI2-TIMEP2

C***Assign LAMSQ at the reference energy.

      DO 29 I=1,NDIM
   29 LAMSQ(I,1)=LAMSQE(I)

C***Assign LAMSQ for TYPE2 second energies.
C***If(.not.LTYPE2) we go around the next section of code.
C***In it we modify the diagonal D matrix elements by adding energy
C***corrections for the second and subsequent energies. Remember that
C***the D matrix elements are (potential matrix elements) + (centrifugal
C***barrier) + E total - E internal; here we just adjust ETOT.

      IF(.NOT.LTYPE2)GO TO 33

      DO 404 J=2,NE
      JM1=J-1
      DO 404 I=1,NDIM
      LAMSQ(I,J)=LAMSQ(I,JM1)+EVADD(J)
  404 CONTINUE 
      GO TO 33

C***Not at reference energy.
C***We reach this portion of code only for NOT LTYPE2 second energy
C***runs, that is for later-date second energy runs
C***and for multiple basis set runs.

   30 IF(.NOT.LPR(1))GO TO 31

C***Assign and write starting time for run.

      CALL SECOND(TIME1)
      WRITE(6,1860)TIME1

C***Again EEV(IE) is the energy of the IE'th run in electron volts.

   31 EEV(IE)=E(IE)*27.21161
      WRITE(6,1910)E(IE),EEV(IE)
      NOPEN(1)=0
 1910 FORMAT(///1X,6HEnergy,1X,1PE15.8,1X,9Hhartree =,1X,E15.8,1X,2HeV)

C***Calculate corrections.
C***ECORR is the energy correction, EVADD includes the reduced mass,
C***and IMADD corrects for the orbital angular momentum (which may
C***differ).  Note that if LCENT is true then the OAM are all zero
C***so that AMADD should not make any difference.

      ECORR=E(1)-E(IE)
      EVADD(1)=TMU*ECORR
      IMADD=OAM(IE)*(OAM(IE)+1)-OAM(1)*(OAM(1)+1)
      ICENT2(IE)=OAM(IE)*(OAM(IE)+1)
      IF(LCENT)GO TO 131
      DO 132 IJP=1,NDIM
  132 CENT(IJP)=DBLE(ICENT2(IE))*YTMU
  131 AMADD=DBLE(IMADD)

C***Calculate LAMSQ for a new energy.

      DO 32 I=1,NDIM
   32 LAMSQ(I,1)=LAMSQE(I)+EVADD(1)

C***Set the correct starting distance for non-LTYPE2 second energy runs.

      DO 41 INR=1,ISMAX
      IF(RSTART(IE).GT.RABC(INR))GO TO 41
      LTESTL=.FALSE.
      INR1=INR-1
      R0=RABC(INR)-H(INR)*.5D0
      GO TO 33
   41 CONTINUE

C***All energies.

   33 LPR2=(LEREF.AND.LPR(6)).OR.LPR1

C***Reset INR if necessary (do it for not LTYPE2 second energy runs).

      IF(INR1.EQ.0)INR=1

C***Set the total angular momentum to OAM of this energy if all L'S =0.

      IF(.NOT.LCENT)JTOT=OAM(IE)

C***Write out starting distance, OAM at this energy and JTOT, and the
C***centers of the first and second sectors at this energy.

      WRITE(6,1920)RABC(INR),OAM(IE),JTOT
      WRITE(6,1930)H(INR1+1),H(INR1+2)
      IF(.NOT.LPR(2)) GO TO 37
      WRITE(6,1940)RASYE
      WRITE(6,1840)(LAMSQ(I,1),I=1,NDIM)
 1930 FORMAT(/1X,26HStepsize in first sector =,1PE13.6,/1X,
     127HStepsize in second sector =,E13.6/)
 1920 FORMAT(/1X,23HCenter of first sector=,1PE15.7/1X, 7HOrbital,
     119H angular momentum =,I5,1X,6HJTOT =,I5)
 1940 FORMAT(1X,41HAsymptotic diagonalized D matrix elements/
     11X,21Hevaluated at RASYE = ,1PE12.6)
 1840 FORMAT(1X,1P,4E16.9)
   37 CALL SECOND(TIMEP1)

C***Calculate quantities for small R boundary conditions.
C***R0 is the location of the left hand side of the first sector.

      CALL POT(R0,NDIM,ID,BPOT,TEMPBC(1,1))
      CALL SECOND(TIMEP2)
      TIMEP=TIMEP+TIMEP2-TIMEP1

C***Calculate the D matrix elements at the small R boundary.

      YIT=1.0D0/R0**2
      DO 390 I=1,NDIM
      VCENT(I)=CENT(I)*YIT+EI(I)-E(IE)
  390 CONTINUE
      VSUM=0D0
      DO 38 I=1,NDIM
      TEMPBC(I,I)=TEMPBC(I,I)+VCENT(I)
      DMI(I)=TEMPBC(I,I)
      VSUM=VSUM+ABS(TEMPBC(I,1))
   38 CONTINUE
      CALL SECOND(TIMDI1)

C***Either Cray SCILIB version of EISPACK RS or EV2RSP is called 
C***depending on the auxilliary files compiled.
C***Subroutine EV2RSP is an alternative routine based on Stephen Elbert's
C***EVVRSP.

      CALL EV2RSP(0,NDIM,NDIM,NBARX,IDX,PCDMT,TEMP4,ISCR,LAM0SQ,        5M9R94
     1            TEMPBC,0,IERR)
      CALL RS(IDX,NDIM,TEMPBC,LAM0SQ,1,TEMPBC,U,U(1,2),IERR)

      DO 391 I=1,NDIM
      LAM0SQ(I)=TMU*LAM0SQ(I)
  391 CONTINUE
      CALL SECOND(TIMDI2)
      TIMEDI=TIMEDI+TIMDI2-TIMDI1

C***Write out error message and stop in case of problems.

      IF(IERR.NE.0)WRITE(6,1950)IERR
 1950 FORMAT(1X,28HWhen diagonalizing D IERR is,I10,14Hat R0 boundary)
      IF(IERR.NE.0)STOP 5800

C***Skip around the next step if this is an LTYPE2 second energy run.
C***Only do it if this is a later-date second energy run.

      IF(LTYPE2)GO TO 405

C***Calculate number of asymptotically open channels, ICT.

      ICT(1)=0
      DO 46 IG=1,NDIM
   46 LMB0SQ(IG,1)=LAM0SQ(IG)+AMADD*YIT
      DO 40 I=1,NDIM
      IF(LAMSQ(I,1).GT.0.0D0)GO TO 42
   40 ICT(1)=ICT(1)+1
   42 WRITE(6,1960)ICT(1)
 1960 FORMAT(1X,44HNumber of asymptotically open channels (ICT),I6)

C***This line skips around the code which counts asymptotially open
C***channels for LTYPE2 second energy runs.

      GO TO 410

C***Now set LMB0SQ and count open channels for TYPE2 second energies.

  405 DO 406 I=1,NDIM
  406 LMB0SQ (I,1)=LAM0SQ(I)
      DO 407 J=2,NE
      JM1=J-1
      DO 407 I=1,NDIM
  407 LMB0SQ (I,J)=LMB0SQ(I,JM1)+EVADD(J)-OAM(JM1)*(OAM(JM1)+1.0)*YIT
     1+OAM(J)*(OAM(J)+1.0)*YIT
      DO 408 J=1,NE
      ICT(J)=0
      DO 408 I=1,NDIM
  408 IF(LAMSQ(I,J).LT.0.0)ICT(J)=I
      WRITE(6,1970)(ICT(I),I=1,NE)
 1970 FORMAT(1X,54HNumber of asymptotically open channels for each energ
     1y,/(1X,10I5))

C***Vary value of stepsize criteria.

  410 DO 200 IEPS=1,NEPS
      EPSLA=EPSA(IEPS)
      EPSLB=EPSB(IEPS)
      EPSLC=EPSC(IEPS)
      HMINLA=HMINA(IEPS)
      HMINLB=HMINB(IEPS)
      HMINLC=HMINC(IEPS)
      HMAXLA=HMAXA(IEPS)
      HMAXLB=HMAXB(IEPS)
      HMAXLC=HMAXC(IEPS)
      HMAX=HMAXLA
      HMIN=HMINLA
      REPSLA=REPSA(IEPS)
      REPSLB=REPSB(IEPS)
      WRITE(6,1980)REPSLA,EPSLA,HMINLA,HMAXLA,REPSLB,REPSLA,EPSLB,
     1HMINLB,HMAXLB,REPSLB,EPSLC,HMINLC,HMAXLC
 1980 FORMAT(/1X,43HStepsize criteria EPS, Minimum and Maximum /
     18X,3HFor,5X,6HR .LT.,1X,1PE10.3,2H, ,5HEPS =,G8.2,7H, MIN =,
     2G8.2,8H & MAX =,1PG8.2/,E10.3,12H .GT. R .GE.,1X,E10.3,2H, ,
     35HEPS =,G8.2,7H, MIN =,1PG8.2,8H & MAX =,1PG8.2/16X,6HR .GE.,
     41X,E10.3,2H, ,5HEPS =,G8.2,7H, MIN =,1PG8.2,8H & MAX =,1PG8.2)

C***R matrix propagation.
C***Start in sector one and continue to asymptotic region.
      IS=1
      IF(IE.GT.1)IS=INR1+1

C***Set NEWRED for both TYPE2 and non TYPE2 runs.

      DO 411 I=1,NEB
  411 NEWRED(I)=PDIM

C***Initialize logical variables.
C***LENTER is set to false for each new propagator and is set true
C***in the first call to subroutine GNSCAT.
C***LCONV is set to false for each new propagator and is set true
C***when the calculations are converged or for RASY.GE.RMAX..
C***LASY is defined by (RABC(IS).GT.RBIG), i.e. when one can drop
C***uncoupled channels from the propagation.
C***LRS is associated with manual convergence with respect to R which
C***is no longer used.
C***LEFACT governs the printing out of EFACT when it is available.
C***EFACT is the sum of the absolute values of the (off diagonal)
C***potential elements linking channel 1 to all other channels; this
C***value is not available if the large RABC region has not been
C***reached or if the D matrix has not yet been diagonalized.
C***LTD determines whether we dump to tape.
C***LLAST defines whether or not we are in the last sector.

      LENTER=.FALSE.
      LCONV=.FALSE.
      LASY=.FALSE.
      LRS=.FALSE.
      LEFACT=.FALSE.
      LTD=.FALSE.
      LTID=.FALSE.
      LLAST=.FALSE.

C***KS is used as a counter to see whether or not we need to write
C***restart information to for017 or for018 as we do every ISAVE sectors.

      KS=0
      TIMEE=0.0
   60 LSTOP1=IS.GE.NSECT3
      CALL SECOND(TIMTMP)
      TIMEE=TIMEE+TIMTMP-TIME1
      TIME1=TIMTMP

C***We do the next section of code only if this is a restart run.
      IF(IREST.EQ.0)GO TO 510

C***If restart information is to be read in from unit 17 then IREST
C***should have been 17; if restart information is to be read from
C***unit 18 then IREST should have been 18.  If IREST is neither one
C***of these values then a warning message will be printed and the
C***program will attempt to read from FORTRAN unit 17.

      IF((IREST.NE.17).AND.(IREST.NE.18))WRITE(6,1990)
 1990 FORMAT(/1X,50HYou have chosen an invalid unit number from which ,
     128Hto read restart information./1X,27HThe program will attempt to,
     227H read from unit 17 instead./)

C***Now reset irest to 17 if an invalid entry is read in.

      IF((IREST.NE.17).AND.(IREST.NE.18)) IREST=17
      IF(IREST.EQ.17)OPEN(UNIT=FU17,FILE='fort.17',STATUS='OLD',
     1FORM='UNFORMATTED')
      IF(IREST.EQ.18)OPEN(UNIT=FU18,FILE='fort.18',STATUS='OLD',
     1FORM='UNFORMATTED')
      READ(IREST)LTHREW
      IF(LTHREW)THEN
      WRITE(6,2000)
 2000 FORMAT(1X,'LTHREW is true, so the propagation is done.')
      STOP 5900
      ELSE
      WRITE(6,2010)
 2010 FORMAT(1X,'LTHREW is false, so the propagation must continue.')
      ENDIF
      READ(IREST)LEREF,LPD,LRS,IS,ISMAX,KS,NEWRED,(IJ,NRED(I),I=1,NEB),
     1ILRS,KTDUMP
      WRITE(6,2020)
 2020 FORMAT(1X,13HRead record 2)
      READ(IREST)(NRD(I),I=1,IS)
      WRITE(6,2030)
 2030 FORMAT(1X,13HRead record 3)
      IF(LR4MT)READ(IREST)YITPST,((EFCNM1(J,I),J=1,NDIM),I=1,NDIM),
     1(((JJ,II,KK,R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     2(DMIM1(I),I=1,NDIM),HMAX,(H(I),I=1,IS),(RABC(I),I=1,IS),
     3((EVALE(I,J),I=1,NDIM),J=1,1),(TIMC(I),I=1,17),TIMEE,TIMER,TIMEG,
     4TIMET,TIMEP,TIMED,TIMEDI
      IF(.NOT.LR4MT)READ(IREST)YITPST,((EFCNM1(J,I),J=1,NDIM),I=1,NDIM),
     1(((JJ,II,KK,R1MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     2(((JJ,II,KK,R2MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     3(((JJ,II,KK,R3MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     4(((JJ,II,KK,R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     5(DMIM1(I),I=1,NDIM),HMAX,(H(I),I=1,IS),(RABC(I),I=1,IS),
     6((EVALE(I,J),I=1,NDIM),J=1,1),(TIMC(I),I=1,17),TIMEE,TIMER,TIMEG,
     7TIMET,TIMEP,TIMED,TIMEDI

C***If NPROP.NE.2 then we need the R4MTM1 (for LR4MT) or the R1MTM1,
C***R2MTM1, R3MTM1, R4MTM1 for (.NOT.LR4MT).  But these are just the
C***R4MT or R1MT, R2MT, R3MT, R4MT we just read in.  Transfer the
C***contents of the arrays read in.

       IF(NPROP.NE.2)THEN 
         IF(LR4MT)THEN
           DO 9444 K=1,NEB
              DO 9444 J=1,NRED(K)
                 DO 9444 I=1,NRED(K)
                     R4MTM1(J,I,K)=R4MT(J,I,K)
 9444      CONTINUE
         ELSE
           DO 9445 K=1,NEB
              DO 9445 J=1,NRED(K)
                 DO 9445 I=1,NRED(K)
                    R1MTM1(J,I,K)=R1MT(J,I,K)
                    R2MTM1(J,I,K)=R2MT(J,I,K)
                    R3MTM1(J,I,K)=R3MT(J,I,K)
                    R4MTM1(J,I,K)=R4MT(J,I,K)
 9445      CONTINUE
         END IF
       END IF

C***Write additional information to FORTRAN unit 6.

      WRITE(6,2040)IS,RABC(IS)
 2040 FORMAT(/,1X,34HThis restart run begins at sector ,I4,
     115H where RABC is ,F9.4/)

C***Reset IRESTART and initialize ISECT.

      IREST=0
      ISECT=0

C***If we don't want to write to tape (can only be true if IROWS.NE.1
C***or when IROWS.NE.4) skip to line 516.

      IF(LDWTT)GO TO 516
      IADD=1
      IF(.NOT.LEREF)IADD=0
      IF(KTDUMP.LT.1+IADD)GO TO 516
      DO 517 I=1,KTDUMP
  517 CALL TLOAD(ID,IS,TAUE,TINVE,LNOINV)
      IF(IADD.EQ.0)GO TO 516
      BACKSPACE(8)
      GO TO 516
  510 ISECT=ISECT+1

C***If we are not at a sector where we write out restart information
C***skip the next section of code.
      IF(ISECT.NE.ISAVE)GO TO 516
      ISECT=0
      KTDUMP=KTDUMP+1
      REWIND (ISAVU)
      IF(ISAVU.EQ.17)CLOSE(UNIT=FU17)
      IF(ISAVU.EQ.18)CLOSE(UNIT=FU18)
      OPEN(UNIT=FU17,FILE='fort.17',STATUS='UNKNOWN',
     1FORM='UNFORMATTED')
      OPEN(UNIT=FU18,FILE='fort.18',STATUS='UNKNOWN',
     1FORM='UNFORMATTED')
      IF(ISAVU.EQ.17)WRITE(17)LTHREW
      IF(ISAVU.EQ.17)WRITE(17)LEREF,LPD,LRS,IS,ISMAX,KS,NEWRED,
     1(I,NRED(I),I=1,NEB),ILRS,KTDUMP
      IF(ISAVU.EQ.17)WRITE(17)(NRD(I),I=1,IS)
      IF((ISAVU.EQ.17).AND..NOT.LR4MT)WRITE(17)
     1YITPST,((EFCNM1(J,I),J=1,NDIM),I=1,NDIM),
     2(((J,I,K,R1MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     3(((J,I,K,R2MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     4(((J,I,K,R3MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     5(((J,I,K,R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),(DMIM1(I),
     6I=1,NDIM),HMAX,(H(I),I=1,IS),(RABC(I),I=1,IS),
     7((EVALE(I,J),I=1,NDIM),J=1,1),(TIMC(I),I=1,17),TIMEE,TIMER,TIMEG,
     8TIMET,TIMEP,TIMED,TIMEDI
      IF((ISAVU.EQ.17).AND.LR4MT)WRITE(17)
     1YITPST ,((EFCNM1(J,I),J=1,NDIM),I=1,NDIM),
     2(((J,I,K,R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),(DMIM1(I),
     3I=1,NDIM),HMAX,(H(I),I=1,IS),(RABC(I),I=1,IS),
     4((EVALE(I,J),I=1,NDIM),J=1,1),(TIMC(I),I=1,17),TIMEE,TIMER,TIMEG,
     5TIMET,TIMEP,TIMED,TIMEDI
      IF(ISAVU.EQ.18)WRITE(18)LTHREW
      IF(ISAVU.EQ.18)WRITE(18)LEREF,LPD,LRS,IS,ISMAX,KS,NEWRED,
     1(I,NRED(I),I=1,NEB),ILRS,KTDUMP
      IF(ISAVU.EQ.18)WRITE(18)(NRD(I),I=1,IS)
      IF(ISAVU.EQ.18.AND..NOT.LR4MT)WRITE(18)
     1YITPST,((EFCNM1(J,I),J=1,NDIM),I=1,NDIM),
     2(((J,I,K,R1MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     3(((J,I,K,R2MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     4(((J,I,K,R3MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),
     5(((J,I,K,R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),(DMIM1(I),
     6I=1,NDIM),HMAX,(H(I),I=1,IS),(RABC(I),I=1,IS),
     7((EVALE(I,J),I=1,NDIM),J=1,1),(TIMC(I),I=1,17),TIMEE,TIMER,TIMEG,
     8TIMET,TIMEP,TIMED,TIMEDI
      IF(ISAVU.EQ.18.AND.LR4MT)WRITE(18)
     1YITPST,((EFCNM1(J,I),J=1,NDIM),I=1,NDIM),
     2(((J,I,K,R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),K=1,NEB),(DMIM1(I),
     3I=1,NDIM),HMAX,(H(I),I=1,IS),(RABC(I),I=1,IS),
     4((EVALE(I,J),I=1,NDIM),J=1,1),(TIMC(I),I=1,17),TIMEE,TIMER,TIMEG,
     5TIMET,TIMEP,TIMED,TIMEDI

C***Write the sector number and R at which restart information was
C***saved.

      WRITE(6,2050)IS,RABC(IS),ISAVU
 2050 FORMAT(1X,35H Restart quantities saved at sector,I6,1X,14Hwhere RA
     1BC is ,F9.4,8H on unit,I5,1H.)

C***Close the appropriate restart unit (ISAVU).

      IF(ISAVU.EQ.17)CLOSE(UNIT=FU17)
      IF(ISAVU.EQ.18)CLOSE(UNIT=FU18)

C***Switch the FORTRAN unit number to which restart information is
C***written (it alternates between units 17 and 18).

      ITEMP=ISAVUP
      ISAVUP=ISAVU
      ISAVU=ITEMP
  516 CONTINUE
C***If we wish to write at ISAVV (we usually don't) we do it here.

      IF(MOD(IS,ISAVV).EQ.0)THEN
         IF((IUNXX.LT.19).AND.(IUNXX.NE.0))THEN
            WRITE(6,2060)ISAVV,IUNXX
            IUNXX=19
         ELSE IF(IUNXX.EQ.0)THEN
            WRITE(6,2070)ISAVV
            IUNXX=6
         ELSE IF(IUNXX.GT.19)THEN
            WRITE(6,2080)ISAVV,IUNXX
         END IF
  555    IF(IFTXX.NE.0)THEN
            IF(IUNXX.EQ.6)THEN
               WRITE(6,2090)
               IFTXX=0
               GO TO 555
            END IF
            OPEN(UNIT=FU19,FILE='fort.19',STATUS='NEW',
     1      FORM='UNFORMATTED')
            WRITE(IUNXX)NEB,(NRED(K),K=1,NEB)
            WRITE(IUNXX)RABC(I),TIMEE,TIMER,TIMEG,TIMET,TIMEP
            WRITE(IUNXX)(((R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),
     1      K=1,NEB)
            STOP 6000
         ELSE
            OPEN(UNIT=FU19,FILE='fort.19',STATUS='NEW',
     1      FORM='FORMATTED')
            WRITE(IUNXX,2100)NEB,(NRED(K),K=1,NEB)
            WRITE(IUNXX,2110)RABC(I),TIMEE,TIMER,TIMEG,TIMET,TIMEP
            WRITE(IUNXX,2120)(((R4MT(J,I,K),J=1,NRED(K)),I=1,NRED(K)),
     1      K=1,NEB)
            STOP 6000
         ENDIF
      END IF
 2060 FORMAT(/1X,10HAt sector ,I4,34H you attempted to write output to ,
     14Hunit,I4,/2X,47Hbut this is invalid. Output will be written to ,
     215Hunit 19 instead)
 2070 FORMAT(/1X,10HAt sector ,I4,33H output will be written to unit 6)
 2080 FORMAT(/1X,10HAt sector ,I4,31H output will be written to unit,I4)
 2090 FORMAT(/1X,52HYou attempted to write binary output to unit 6 which
     1,/2X,54H is invalid.  Formatted output will be written instead)
 2100 FORMAT(12I4)
 2110 FORMAT(3G20.12)
 2120 FORMAT(3G20.12)

C***The lines below tell the code to stop if in the NSECT-1 sector.

      LSTOP2=IS.EQ.NSECT1

C***The line below doesn't let you stop if you are not at the reference
C***energy.

      IF(.NOT.LEREF)GO TO 55
      LLAST=.TRUE.
      GO TO 56
   55 IF(IS.GE.ISMAX)LLAST=.TRUE.
      IF(IS.LE.ISMAX)GO TO 56
      LEREF=.TRUE.
      IF(MOD(KS,NSAVE).EQ.0)KS=0
   56 CONTINUE

C***LEARLY is not related to which sector we are in but only determines
C***whether the title for the sector has been printed in the calling
C***program.

      LEARLY=.FALSE.

C***Write out sector information if we want it.

      IF(.NOT.(LPR(4).AND.LEREF))GO TO 57
      CALL TITLES(IS,2,DUM,ID,NSECT,LGS,RABC)
      LEARLY=.TRUE.

C***NEWRED may change in call to subroutine DROP.
C***Re-assign the number of dimensions propagated for each energy/basis
C***in case any of these have dropped channels in the last sector.

   57 DO 412 I=1,NEB
  412 NRED(I)=NEWRED(I)
      IF(((NE.EQ.1).AND.(NPD.EQ.1)).AND.((IROWS.NE.1).AND.(IROWS.NE.4)))
     1PDIM=NRED(1)

C***Check whether the large RABC region has been reached.

      YIT=1.0D0/RABC(IS)**2
      LASY=RABC(IS).GT.RBIG
      IF(.NOT.LEREF)GO TO 66

C***Reference energy.
C***Assign NRD.

      NRD(IS)=NRED(1)

C***Calculate D MATRX and store unpacked UPCDMT, store diagonal elements
C***in DMI.

      CALL SECOND(TIMEP1)
      CALL POT(RABC(IS),NDIM,ID,BPOT,TEMPBC(1,1))
      CALL SECOND(TIMEP2)
      TIMEP=TIMEP+TIMEP2-TIMEP1
      DO 392 I=1,NDIM
      VCENT(I)=CENT(I)*YIT+EI(I)-E(IE)
  392 CONTINUE

C***Create D matrix from diagonal elements of interaction and VCENT.
C***Store diagonal elements in DMI(I).

      VSUM=0D0
      DO 59 I=1,NDIM
      TEMPBC(I,I)=TEMPBC(I,I)+VCENT(I)
      DMI(I)=TEMPBC(I,I)
      VSUM=VSUM+ABS(TEMPBC(I,1))
   59 CONTINUE

C***Write out unpacked interaction matrix if desired.

      IF(.NOT.LPR(4))GO TO 62
      WRITE(6,2130)
      WRITE(6,2140)((I,J,TEMPBC(I,J),I=1,NDIM),J=1,NDIM)
 2130 FORMAT(1X,39HElements of unpacked interaction matrix)
 2140 FORMAT(1X,3(2I4,1X,1PE12.5,1X))

C***Calculate the eigenvalues and eigenfunctions
C***for single channel.

      NOPEN(1)=0
      IF(NDIM.GT.1)GO TO 62
      EVALI(1)=TEMPBC(1,1)
      IF(EVALI(1).LT.0D0)NOPEN(1)=1
      TEMPBC(1,1)=1.0D0
      GO TO 64

C***For many channels.

   62 CONTINUE

C***Either Cray SCILIB version of EISPACK RS or EV2RSP is called 
C***depending on the auxilliary files compiled.
C***Subroutine EV2RSP is an alternative routine based on Stephen Elbert's
C***EVVRSP.

      CALL EV2RSP(0,NDIM,NDIM,NBARX,IDX,PCDMT,TEMP4,ISCR,EVALI,         5M9R94
     1            TEMPBC,0,IERR)
      CALL RS(IDX,NDIM,TEMPBC(1,1),EVALI,1,TEMPBC,U,U(1,2),IERR)

C***Stop execution on error exit.

      IF(IERR.NE.0)WRITE(6,2150)IERR,IS
 2150 FORMAT(1X,7HIERR is,I10,9Hin sector,I10)
      IF(IERR.NE.0)STOP 6100
      DO 201 I=1,NDIM
      EVALI(I)=EVALI(I)*TMU
      IF(EVALI(I).GT.0D0)GO TO 201
      NOPEN(1)=I
  201 CONTINUE

C***Keep track of time to diagonalize.

      CALL SECOND(TIMDI2)
      TIMEDI=TIMEDI+TIMDI2-TIMEP2

C***In the large RABC region check for degenerate eigenvalues
C***and adjust eigenfunctions if necessary, then check phase
C***of eigenfunctions and make the largest component of each
C***eigenfunction a positive number if it is not one already.

      CALL SECOND(TIMET1)
      IF(.NOT.LASY.OR.LTASY)GO TO 965
      DO 964 J=1,NDIM
      DIJ=0.0D0
      ADIJ=0.0D0
      MAXI=0
      DO 963 I=1,NDIM
      AEIJ=ABS(TEMPBC(I,J))
      IF(ADIJ.GT.AEIJ)GO TO 963
      DIJ=TEMPBC(I,J)
      ADIJ=ABS(DIJ)
      MAXI=I
  963 CONTINUE
      IF((DIJ/ADIJ).GT.0)GO TO 964
      DO 966 K=1,NDIM
  966 TEMPBC(K,J)=-TEMPBC(K,J)
  964 CONTINUE
  965 CONTINUE
      CALL SECOND(TIMET2)
      TIMC(3)=TIMC(3)+TIMET2-TIMET1

C***Place eigenvalues in array for use at other energies.
C***Don't store in EVALE if they will never be needed.

   64 IF(LDWTT)GO TO 68

      DO 65 I=1,NDIM
   65 EVALE(I,IS)=EVALI(I)-EVADD(1)-AMADD*YIT
      GO TO 68

C***Not at reference energy.
C***Correct previously calculated eigenvalues.

   66 CONTINUE
      DO 67 I=1,NDIM
   67 EVALI(I)=EVALE(I,IS)+EVADD(1)+AMADD*YIT

C***Count the number of open channels based on corrected eigenvalues.

      NOPEN(1)=0
      DO 202 I=1,NDIM
      IF(EVALI(I).GT.0D0)GO TO 202
      NOPEN(1)=I
  202 CONTINUE

C***At all energies.

   68 CONTINUE

C***Small R region.

      IF(LASY)GO TO 80
      IF(.NOT.LPD)GO TO 85

C***Write sector information only if not previously written.

      LPR2=(LPR2.AND..NOT.LEARLY)
      IF(.NOT.LPR2)GO TO 75
      CALL TITLES(IS,2,DUM,ID,NSECT,LGS,RABC)
      LEARLY=.TRUE.

C***Write out eigenvalues if desired.

   75 IF(.NOT.LPR(5))GO TO 78
      WRITE(6,2160)
      WRITE(6,1840)(EVALI(I),I=1,NDIM)
 2160 FORMAT(1X,11HEigenvalues)
   78 IF(.NOT.LEREF)GO TO 85

C***Write out eigenfunctions (for reference energy only-see line above)
C***if desired.

      IF(.NOT.LPR(6))GO TO 90
      WRITE(6,2170)
 2170 FORMAT(1X,21HColumn eigenfunctions)
      CALL PRNTM2(TEMPBC,NDIM,NDIM,ID)
      GO TO 90

C***Asymptotic region.
C***Reference energy.

   80 IF(.NOT.LEREF)GO TO 85
      EFACT=VSUM
      LEFACT=.TRUE.
      IF(LRS)GOTO 87
      LRS=EFACT.LT.CRIRS
      IF(LRS)ILRS=IS
      GO TO 87

C***Not at reference energy.

   85 IF(IS.GE.ILRS)LRS=.TRUE.
      LEFACT=.FALSE.
      IF(.NOT.LRS)GOTO 95

C***All energies.
C***Print out sector information if desired and if not already done.

   87 IF(.NOT.LEARLY.AND.LPSINF)CALL TITLES(IS,2,EFACT,ID,NSECT,LGS,
     1RABC)
      LEARLY=.TRUE.
      IF (.NOT.LEREF) GO TO 95

C***Reference energy and basis.
C***Update KS and save the current value of KS in KKS (KS is the
C***counter for which sector we print in).

   90 KS=KS+1
      KKS=KS
      CALL SECOND(TIMET1)

C***Calculate TAUI and TINVI.
C***Subroutine TAUMTS calculates the matrices TAUI(PD,PD) and TINVI(PD,
C***PD)
      CALL TAUMTS(IS,NDIM,PDIM,NRED,LPR(7),ID,1,NPROP ,KKS,TEMPBC,
     $            TAUE(1,1,KKS),TINVE(1,1,KKS),U,EFCNM1,INR1,LDWTT,
     $            LEREF,EPSMV,IPIV)

      CALL SECOND(TIMET2)
      TIMET=TIMET+TIMET2-TIMET1

C***LTD determines whether we call TDUMP which writes TAUI and TINVE to
C***FORTRAN unit 8. It is set to .false. right after every call to TDUMP.

      LTD=.TRUE.

C***INE is a counter to tell which energy of a multiple-energy run we
C***are doing.

      INE=1
      IF(MOD(KS,NSAVE).NE.0)GO TO 540
      IF(LDWTT)GO TO 130
      CALL TDUMP(1,ID,TAUE,TINVE,LNOINV)
  130 LTD=.FALSE.
      KS=0
      GO TO 540

C***Not at reference energy or not at the first basis.

   95 KS=KS+1

C***Note that just above (line 95) we updated the counter for comparing
C***sector to NSAVE, but not the sector.  Therefore compare KS-1 to
C***NSAVE to see if we need to read anything in.  From there down to
C***line ending in GO TO 96 read in TAUE and TINVE until the current
C***sector (NSAVE is the number of sectors in memory simultaneously).

      IF(MOD(KS-1,NSAVE).NE.0)GO TO 97
   96 CALL TLOAD(ID,IS,TAUE,TINVE,LNOINV)
      IF(LTESTL)GO TO 92
      INR=INR-NSAVE
      IF(INR.GT.0)GO TO 96
      KS=NSAVE+INR
      LTESTL=.TRUE.
      GO TO 97
   92 KS=1
   97 KKS=KS
      INR2=3
      IF((IS-INR1).EQ.1)INR2=1
      CALL SECOND(TIMET1)

C***Call TAUMTS for .NOT.LTYPE2 second energy runs where INR2 tells
C***TAUMTS which sector we are in.

      CALL TAUMTS(IS,NDIM,PDIM,NRED,LDUM,ID,INR2,NPROP ,KKS,TEMPBC,
     $            TAUE(1,1,KKS),TINVE(1,1,KKS),U,EFCNM1,INR1,LDWTT,
     $            LEREF,EPSMV,IPIV)
      INE=1
      CALL SECOND(TIMET2)
      TIMET=TIMET+TIMET2-TIMET1

C***Compare the number of channels to be propagated at the first energy
C***(which is also the dimension of many matrices) to the number of
C***channels to be propagated at this sector.
C***Note that this section is never even reached for first energies.

      IF(NRED(1).EQ.NRD(IS))GO TO 540
      CALL SECOND(TIMET1)
      INR2=2
      CALL TAUMTS(IS,IDUM,IDUM1,NRED,LPR(7),ID,INR2,NPROP ,KKS,TEMPBC,
     1TAUE(1,1,KKS),TINVE(1,1,KKS),U,EFCNM1,INR1,LDWTT,LEREF,EPSMV,IPIV)
      CALL SECOND(TIMET2)
      TIMET=TIMET+TIMET2-TIMET1
      LTID=.TRUE.
      IF(MOD(KS,NSAVE).NE.0)GO TO 540
      LTID=.FALSE.

C***All energies and bases.
C***All energies.
C***Calculate P matrices P1 and P3INV.
C***Calculate small R matrices.
C***Calculate R matrices.

  540 CONTINUE
      CALL SECOND(TIMES1)

C***Test the symmetry of the R4MT matrix.

      IF(LPSYM(1))THEN
      DO 547 INE1=1,NEB
      IF(IS.LE.2)GO TO 587
      RSYM(INE1)=0.0
      DO 548 IROW=1,ICT(INE1)
      DO 548 ICOM=IROW,ICT(INE1)
      RTEMP=MAX(R4MT(IROW,ICOM,INE1),R4MT(ICOM,IROW,INE1))
      IF(ABS(RTEMP).LT.1.0D-20)GO TO 548
      RSYM(INE1)=RSYM(INE1)+ABS((R4MT(IROW,ICOM,INE1)-
     1R4MT(ICOM,IROW,INE1))/MAX(R4MT(IROW,ICOM,INE1),
     2R4MT(ICOM,IROW,INE1)))
  548 CONTINUE
      RSYM(INE1)=RSYM(INE1)/DBLE(ICT(INE1)*(ICT(INE1)+1)/2)
  547 CONTINUE
      WRITE(6,2180)IS,(RSYM(INE1),INE1=1,NEB)
 2180 FORMAT(1X,35HSymmetry checks for R4MT at sector ,I5,/
     1,(1X,1P,10E10.2))
  587 CONTINUE
      END IF
      CALL SECOND(TIMES2)
      TIMC(9)=TIMC(9)+TIMES2-TIMES1
  100 CALL SECOND(TIMER1)

C***Subroutine RPROP does the actual R-matrix propagation.

      CALL RPROP(NRED(INE),IS,ID,LPR(8),LPR(9),LR4MT,NPROP,KKS,NSECT,
     1EVALI,P1,P3INV,TAUE(1,1,KKS),TINVE(1,1,KKS),R4MT(1,1,INE),U,
     $H,INR1,LPSYM,TEMPBC,TEMP2,ETA,EPSMV,NEWRED(INE),EPSRED,
     $     RABC(IS),LASY,ICT(INE),R1MT(1,1,INE),R1MTM1(1,1,INE),
     $R2MT(1,1,INE),R2MTM1(1,1,INE),R3MT(1,1,INE),R3MTM1(1,1,INE),
     $R4MTM1(1,1,INE),TEMPR1,IPIV)
      CALL SECOND(TIMER2)
      TIMER=TIMER+TIMER2-TIMER1

C***Check for TYPE2 second energies, and propagate them.

      IF(.NOT.LTYPE2)GO TO 413
      IF(NRED(INE).EQ.NEWRED(INE))GO TO 418
      WRITE(6,2190)INE
 2190 FORMAT(1X,52HThe previously dropped channel was for energy number,
     1I5)

C***Check symmetry on TAUI matrix.

      CALL SECOND(TIMES1)
      IF(LPSYM(2))THEN
      DO 552 I1=1,NRED(INE)
      DO 552 I2=1,NRED(INE)
      TEMPBC(I1,I2)=TEMPBC(I1,I2)+TAUE(I1,I2,KKS)*TAUE(I1,I2,KKS)
 552  CONTINUE
      TOFF=0.0
      TDIAG=0.0
      DO 553 IT1=1,NRED(INE)
      DO 553 IT2=1,NRED(INE)
      IF(IT1.EQ.IT2)GO TO 554
      TOFF=TOFF+ABS(TEMPBC(IT2,IT1))
      GO TO 553
  554 TDIAG=TDIAG+TEMPBC(IT1,IT1)
  553 CONTINUE
      TOFF=TOFF/DBLE(NRED(INE)*(NRED(INE)-1))
      TDIAG=TDIAG/DBLE(NRED(INE))
      WRITE(6,2200)INE,TDIAG,TOFF
 2200 FORMAT(/1X,35HSymmetry checks on TAUI for energy ,I5,1P,2E15.8/)
      END IF
      CALL SECOND(TIMES2)
      TIMC(10)=TIMC(10)+TIMES2-TIMES1
  418 INE=INE+1
      IF(INE.GT.NE)GO TO 413

C***Check for open channels after correction eigenvalues.

      NOPEN(INE)=0
      DO 414 I=1,NDIM
      EVALI(I)=EVALI(I)+EVADD(INE)-OAM(INE-1)*(OAM(INE-1)+1.0)*YIT
     1+OAM(INE)*(OAM(INE)+1.0)*YIT
  414 IF(EVALI(I).LT.0.0)NOPEN(INE)=I
      IF(NRED(INE).EQ.NRED(INE-1))GO TO 100

C***The number of channels propagated at this energy is different
C***then the last, so we must reinvert TAUI.

      CALL SECOND(TIMET1)
      IF(IS.GT.1)THEN
      CALL TAUMTS(IS,NDIM,PDIM,NRED(INE),LPR(7),ID,2,NPROP ,
     1KKS,TEMPBC,TAUE(1,1,KKS),TINVE(1,1,KKS),U,EFCNM1,INR1,LDWTT,
     $LEREF,EPSMV,IPIV)
      END IF
      CALL SECOND(TIMET2)
      TIMET=TIMET+TIMET2-TIMET1
      GO TO 100
  413 IF(IS.EQ.1)GO TO 586

C***Perform symmetry checks on R4MT.

      IF(LPSYM(3))THEN
      CALL SECOND(TIMES1)
      RSYMMX=0.0
      DO 532 INE=1,NEB
      RSYM(INE)=0.0
      DO 533 IRWO=1,ICT(INE)
      DO 533 ICOM=IRWO,ICT(INE)
      RTEMP=MAX(R4MT(IRWO,ICOM,INE),R4MT(ICOM,IRWO,INE))
      IF(ABS(RTEMP).LT.1.0D-20)GO TO 533
      RSYM(INE)=RSYM(INE)+ABS((R4MT(IRWO,ICOM,INE)-
     1R4MT(ICOM,IRWO,INE))/MAX(R4MT(IRWO,ICOM,INE),R4MT(
     2ICOM,IRWO,INE)))
  533 CONTINUE
      RSYM(INE)=RSYM(INE)/DBLE(ICT(INE)*(ICT(INE)+1)/2)
      RSYMMX=MAX(RSYMMX,RSYM(INE))
  532 CONTINUE
      WRITE(6,2210)(RSYM(INE),INE=1,NEB)
 2210 FORMAT(1X,23HSymmetry checks on R4MT,(1X,1P,10E10.2))
      CALL SECOND(TIMES2)
      TIMC(11)=TIMC(11)+TIMES2-TIMES1
      IF(RSYMMX.GT.RSYMTL)STOP 6200
      END IF
  586 CONTINUE

C***If in the next to last sector there is no need to find next RABC.

      IF(LSTOP2)GO TO 125

C***Reference energy.
C***Calculate the next RABC and stepsize subject to HMAX, HMIN and
C***RCUSP requirements.
C***If not at the reference energy skip this since the stepsizes are
C***determined by the first energy.

      IF(.NOT.LEREF)GO TO 118
      CALL SECOND(TIMET1)

C***If this is the first sector we can skip this because the first and
C***second stepsizes are read in from FORTRAN unit 5.

      IF(IS.EQ.1)GO TO 120
      IN=IS+1

C***If H(IS) is too large set the stepsize to HMAX and continue.

      IF(H(IS).LE.HMAX)GO TO 101
      H(IN)=HMAX
      GO TO 111

C***The section of code through lines 103 set the EPSILONs for the
C***variable stepsize region depending on the center of mass separation.
C***Note that these lines are changed to allow for uniformity across
C***both CRAY and IBM architectures.

  101 IF(RABC(IS)-REPSLA.GE.1.0D-12)GO TO 102
      EPSL=EPSLA
      HMAX=HMAXLA
      HMIN=HMINLA
      GO TO 103
  102 EPSL=EPSLB
      HMAX=HMAXLB
      HMIN=HMINLB
      IF(REPSLB-RABC(IS).GE.1.0D-12)GO TO 103
      EPSL=EPSLC
      HMAX=HMAXLC
      HMIN=HMINLC

C***Decide action depending on location and number of cusps in the
C***potential: IAFTER-1 > 0 means no cusps or after the last cusp,
C***IAFTER-1=0 means in between cusps, and IAFTER-1 < 0 means before
C***the first cusp.

  103 CONTINUE
      IF(IAFTER-1)107,104,106
  104 RDIFF=RABC(IS)-RABC(IS-1)
      SUM=0.0
      DO 420 I=1,PDIM
      S=((DMI(I)-DMIM1(I))/RDIFF)
  420 SUM=SUM+S*S
      SUM=SUM*TMU2
      AN=PDIM
      IF(SUM.LE.1.0D-36)GO TO 421
      H(IN)=EPSL/(SUM/AN)**(1.0/6.0)
      GO TO 422
  421 H(IN)=1.0E6
  422 CONTINUE
      H(IN)=H(IN)*0.5D0
      H(IN)=DMIN1(H(IN),HMAX)
      IF(ICUSP.GE.NCUSP)GO TO 105
      H1=(RCUSP(ICUSP+1)-RCUSP(ICUSP))/3.0D0
      H(IN)=DMIN1(H(IN),H1,HMAX)
  105 IAFTER=2
      GO TO 111
  106 H(IN)=H(IS)
      IAFTER=0
      GO TO 111
  107 RDIFF=RABC(IS)-RABC(IS-1)
      SUM=0.0
      DO 423 I=1,PDIM
      S=((DMI(I)-DMIM1(I))/RDIFF)
  423 SUM=SUM+S*S
      SUM=SUM*TMU2
      AN=PDIM
      IF(SUM.LE.1.0D-36)GO TO 424
      H(IN)=EPSL/(SUM/AN)**(1.0/6.0)
      GO TO 425
  424 H(IN)=1.0E6
  425 CONTINUE
      H(IN)=DMIN1(HMAX,H(IN))
      H(IN)=DMAX1(HMIN,H(IN))
      IF(ICUSP.GE.NCUSP)GO TO 111
      RB=RABC(IS)+H(IS)*0.5D0
      RC=RCUSP(ICUSP+1)
      HC=RC-RB
      HDIFF=H(IN)-HC
      IF(HDIFF)108,110,109
  108 IF((-HDIFF).GT.(.1*H(IN)))GO TO 111
  109 H(IN)=HC
  110 ICUSP=ICUSP+1
      IAFTER=1
  111 RABC(IN)=RABC(IS)+(H(IN)+H(IS))*0.5D0
      IF(LWSTT)OPEN(UNIT=FU12,FILE='fort.12',STATUS='NEW',
     1FORM='FORMATTED')
      IF(LWSTT)WRITE(12,2220)RABC(IN),H(IN)
      IF(LRSFT)OPEN(UNIT=FU12,FILE='fort.12',STATUS='OLD',
     1FORM='FORMATTED')
      IF(LRSFT)READ(12,2220)RABC(IN),H(IN)
 2220 FORMAT(1X,5(1PE22.15))
      CALL SECOND(TIMET2)
      TIMC(4)=TIMC(4)+TIMET2-TIMET1
      GO TO 120

C***Not at reference energy.

  118 IF(.NOT.LLAST)GO TO 120
      CALL SECOND(TIMEP1)
      CALL POT(RABC(IS),NDIM,ID,BPOT,TEMPBC(1,1))
      CALL SECOND(TIMEP2)
      TIMEP=TIMEP+TIMEP2-TIMEP1
      LDWTT=.FALSE.
      REWIND(UNIT=FU14)
      IROWS=1
      IF(MOD(KS,NSAVE).EQ.0)GO TO 112
      BACKSPACE(8)
  112 CONTINUE
      DO 393 I=1,NDIM
      VCENT(I)=CENT(I)*YIT+EI(I)-E(IE)
  393 CONTINUE
      DO 119 I=1,NDIM
      DMI(I)=TEMPBC(I,I)+VCENT(I)
  119 CONTINUE

C***All energies.
C***Store the quantities needed for calculations in the next sector
C***at the reference energy in each sector: save DMI as DMIM1.
C***The transpose of the eigenvectors was stored in EFCNM1 by TAUMTS.

  120 CALL SECOND(TIMET1)
      DO 520 I=1,NDIM
      DMIM1(I)=DMI(I)
  520 CONTINUE
      CALL SECOND(TIMET2)
      TIMC(5)=TIMC(5)+TIMET2-TIMET1

C***If in the next to last sector there is no need to continue.

      IF(LSTOP1)GO TO 125
      IF((IPMX.GT.1).AND.(IPRINT.GT.1))THEN
       IF(IPRINT.LE.IPMX)THEN
         IF(RABC(IS).GE.RPSM(IPRINT)
     1      .AND.((RABC(IS-2).LT.RPSM(IPRINT-1)).AND.(RABC(IS-1).GE.
     2      RPSM(IPRINT-1))))THEN
            WRITE(6,2230)RPSM(IPRINT),IS,RPSM(IPRINT-1),IS-1
            IPRINT=IPRINT+1
            GOTO 123
         END IF
       END IF
      END IF
 2230 FORMAT(/47HYou wished to perform an asymptotic analysis at,
     122H intermediate distance,F8.2,/10Hat sector ,I4,5H but ,
     258Hthe asymptotic analysis performed at the last intermediate,
     3/10Hdistance, ,F8.2,22H was performed at the ,
     417Hprevious sector (,I4,1H),/30Hso the asymptotic analysis at ,
     549Hthe present intermediate distance will be skipped/)
      IF(IPRINT.LE.IPMX.AND.RABC(IS).GE.RPSM(IPRINT))GO TO 125
      IF(RABC(IS).GT.RMAX)GO TO 125

C***Increment loop.

  123 YITPST=YIT

C***Update the sector index.

      IS=IS+1
      GO TO 60

C***Calculate reactance matrix if criteria are satisfied.

  125 RASY=RABC(IS)+H(IS)*0.5D0
      INE=1
      IF(IPRINT.GT.IPMX)GO TO 1125

C***This line skips around the line to set LPSMH=.TRUE. and the counter
C***for intermediate distances at which to perform the asymptotic
C***analysis if we have not yet reached the next intermediate distance
C***at which scattering information is desired, so LPSMH stays .FALSE.
C***and nothing will be printed out in GNSCAT.

      IF(RABC(IS).LT.RPSM(IPRINT))GO TO 1125
      IPRINT=IPRINT+1
      LPSMH=.TRUE.
 1125 NDR=NRED(INE)
      CALL SECOND(TIMET1)

      IF(LTYPE2)WRITE(6,2240)INE
      IF(LTYPE2)WRITE(15,2240)INE
 2240 FORMAT(1X,37HAsymptotic analysis for energy number,I5)
      CALL SECOND(TIMET2)
      TIMC(6)=TIMC(6)+TIMET2-TIMET1

      JICT=MIN0(ICT(INE),PDIM)

C***Make sure and pass current energy to GNSCAT.

      IF(LTYPE2)THEN
       JNE=INE
      ELSE
       JNE=IE
      END IF
      CALL SECOND(TIMEG1)

C***Subroutine GNSCAT performs the asymptotic analysis.

      CALL GNSCAT(NDR,ID,JICT,R0,RASY,IS,EFACT,NDIM,RMAX,NSECT,
     1R4MT(1,1,INE),AKMT,AFMT,AGMT,AEMT,AHMT,AK0MT,SPRRE,SPRIM,TEMPBC,
     2TEMP2,TEMPR1,R1MT(1,1,INE),R2MT(1,1,INE),R3MT(1,1,INE),U,LGS,
     3IOPT2,IOPT3,LMB0SQ(1,INE),L,LAMSQ(1,INE),RABC,JIN,ETA,JTOT,JNE,
     4OAM,LPSMH,LBGS,NOPEN(INE),LPSINF,LPEPS,EVALI,TEMP3,ISJ,ISJ2,EI,
     5CENT,E(JNE),LPR(4),EFCNM1,IMATCH,TMU,ITPRNT,EFASY,LTASY,ITCOL,
     6BPOT,INE,IPRINT,RPSM,NIND,IND,EPSMAG,EPSPH,LPSYM,IERR,EPSMV,PCDMT,
     7IPIV,TEMP4,EPSDR)

C***Replace the transformed global R matrices with the originals.

      IF(.NOT.LR4MT)THEN
         IF(LTASY)THEN
            DO 184 I=1,IDX
               DO 184 J=1,IDX
                  R1MT(J,I,INE)=R1MTM1(J,I,INE)
                  R2MT(J,I,INE)=R2MTM1(J,I,INE)
                  R3MT(J,I,INE)=R3MTM1(J,I,INE)
                  R4MT(J,I,INE)=R4MTM1(J,I,INE)
 184        CONTINUE
         END IF
       END IF

C***Loop below added for convergence tests.

      IF(LPSMH.AND.INE.EQ.1)THEN
      DO 185 I=1,JICT
      DO 185 J=1,JICT
      SPRRE(J,I)=TEMPBC(J,I)
      SPRIM(J,I)=TEMPBC(J,I+JICT)
  185 CONTINUE
      END IF
      CALL SECOND(TIMEG2)
      TIMEG=TIMEG+TIMEG2-TIMEG1
      IF(.NOT.LTYPE2)GO TO 416
      INE=INE+1
      IF(INE.GT.NE)GO TO 416
      GO TO 1125
  416 CONTINUE

C***The three lines below have been added to insure that the program
C***stops for RABC(IS).GT.RMAX if there are problems in the calculation
C***of the S-matrix and there is an abnormal return.

      IF(RABC(IS).GT.RMAX.AND.IERR.NE.0)WRITE(6,2260)RABC(IS)
 2260 FORMAT(/1X,49HThere were problems with the S matrix calculation,
     120H program stopped at ,/1X,4HR = ,1PE22.15)
      IF(RABC(IS).GT.RMAX.AND.IERR.NE.0)GO TO 138

C***Skip around re-setting LPSMH to .FALSE. if it is already .FALSE..

      IF(.NOT.LPSMH)GO TO 1127
      LPSMH=.FALSE.
 1127 IF(LSTOP2.OR.LCONV)GO TO 138

C***Increment loop if calculations are not to be stopped yet.

      YITPST=YIT
      IS=IS+1
      GO TO 60

C***Calculations are converged or are being stopped.

  138 IF(LTD.AND..NOT.LDWTT)CALL TDUMP(1,ID,TAUE,TINVE,LNOINV)
      LTD=.FALSE.
      REWIND(UNIT=FU8)
      IF(.NOT.LPR(1)) GO TO 140
      CALL SECOND(TIME2)
      TIMED=TIME2-TIME1
      TIMED=TIMEE+TIMED
      WRITE(6,2270)TIME2,TIMED

C***Write out timings for various portions of the code.

 2270 FORMAT(/1X,24HStop calculation at time,F9.3,2X,
     126HTherefore calculation took,F9.3,1X,7Hseconds)
      PTIMER=100.D0*TIMER/TIMED
      PTIMEG=100.D0*TIMEG/TIMED
      PTIMET=100.D0*TIMET/TIMED
      PTIMDI=100.D0*TIMEDI/TIMED
      PTIMEP=100.D0*TIMEP/TIMED
      WRITE(6,2280)TIMER,PTIMER,TIMEG,PTIMEG,TIMET,PTIMET,TIMEDI,PTIMDI
     1,TIMEP,PTIMEP,(PARTIM(II),II=1,NPOT)
 2280 FORMAT(1X,18HTiming information/14X,4HTime,7X,17HPer cent of total
     1/1X,5HRPROP,5X,F9.3,10X,F5.1/1X,7HGNSCAT ,3X,F9.3,10X,F5.1/1X,
     26HTAUMTS,4X,F9.3,10X,F5.1/1X,9HEV2RSP   ,1X,F9.3,10X,F5.1/1X,3HPOT
     3,7X,F9.3,10X,F5.1,/1X,'Potential time(s) ',(1PE15.8))
      WRITE(6,2290)(TIMC(IT),IT=1,8)
 2290 FORMAT(1X,14HRead & setup  ,F11.5,/1X,14HPREPOT & POT  ,F11.5,/1X,
     114HPhase         ,F11.5/1X,14HStepsize      ,F11.5,/1X,
     214HLast          ,F11.5/1X,14HDROP          ,F11.5,/1X,
     314HCalc S matrix ,F11.5/1X,14HEXSCAT        ,F11.5)
      WRITE(6,2300)(TIMC(ITT),ITT=17,18)
 2300 FORMAT(1X,14HLinear Soln   ,F11.5,/1X,16HMatrix multiply ,F11.5)

C***Diagnostic write out.

C     WRITE(6,2310)(TIMC(IT),IT=9,16)

C2310 FORMAT(1X,8HLPSYM(1),F11.5,/1X,8HLPSYM(2),F11.5,/1X,
C    18HLPSYM(3),F11.5/1X,8HLPSYM(4),F11.5,/1X,8HLPSYM(5),F11.5,/1X,
C    28HLPSYM(6),F11.5/1X,38HLPSYM(7),F11.5,1X,8HLPSYM(8),F11.5)

  140 IF(LPR(10).AND.(LSTOP2.OR.(IE.EQ.1.AND.IP.EQ.1)))GO TO 150
      IF(LSTOP2)STOP 6400

C***Remember the LOOP 200 is over stepsize criteria.

      GO TO 200

C***Last energy(or calculation being stopped).
C***Write the stepsize information by sector.

  150 WRITE(6,2320)
 2320 FORMAT(1X,3(6HSector,5X,4HRABC,9X,8HStepsize,8X))
      DO 156 I=1,IS,3
      IP1=I+1
      IP2=I+2
      WRITE(6,2330)I,RABC(I),H(I),IP1,RABC(IP1),H(IP1),
     1IP2,RABC(IP2),H(IP2)
 2330 FORMAT(1X,3(1X,I4,2(2X,1PE13.7),5X))
  156 CONTINUE
      IF(LSTOP2)STOP 6500

C***All energies.
C***Marker for EPSA and EPSB variation.

  200 IPRINT=1

C***Consider calculation at other energies.

      IF(IE.EQ.1)ISMAX=IS
      ISMAX=MAX0(ISMAX,IS)
      LEREF=.FALSE.
      IE=IE+1

C***The line immediately below jumps out of the loop if we have done
C***all of the energies and/or all of the different size basis sets.
C***The line below that sends us back to do the next value if we have
C***not finished.

      IF((IE.GT.NE).AND.(IP.EQ.NPD))GO TO 300
      IF(IE.LE.NE.AND..NOT.LTYPE2)GO TO 30
      IE=1

C***Marker for PDIM variation.

  225 CONTINUE

C***This loop permits one to run several potential approximations for
C***one value of the energy.

  300 IF(.NOT.LSTAP)GO TO 21
      IF(IROWS.EQ.1.OR.IROWS.EQ.4)GO TO 303

C***Open FORTRAN unit 3 and write out the time and the date in formatted form.

      OPEN(UNIT=FU3,FILE='fort.3',STATUS='UNKNOWN',FORM='FORMATTED')
      REWIND(UNIT=FU3)
      WRITE(3,1022)TIMDAT(1:80)

C***Close all FORTRAN units if they have not already been closed.

      CLOSE(UNIT=FU3)
      CLOSE(UNIT=FU4)
      CLOSE(UNIT=FU5)
      CLOSE(UNIT=FU7)
      CLOSE(UNIT=FU8)
      CLOSE(UNIT=FU9)
      CLOSE(UNIT=FU10)
      CLOSE(UNIT=FU11)
      CLOSE(UNIT=FU12)
      CLOSE(UNIT=FU13)
      CLOSE(UNIT=FU14)
      CLOSE(UNIT=FU15)
      CLOSE(UNIT=FU16)
      CLOSE(UNIT=FU17)
      CLOSE(UNIT=FU18)
      CLOSE(UNIT=FU19)

C***STOP 6300 is the marker for normal, successful completion.

      STOP 6300
  303 CONTINUE
      OPEN(UNIT=FU3,FILE='fort.3',STATUS='UNKNOWN',FORM='FORMATTED')
      REWIND(UNIT=FU3)
      WRITE(3,1022)TIMDAT(1:80)
      CLOSE(UNIT=FU3)
      CLOSE(UNIT=FU14)
      OPEN(UNIT=FU14,FILE='fort.14',STATUS='UNKNOWN',FORM='UNFORMATTED')
      REWIND 14
      WRITE(14)ISMAX
      WRITE(14)E(1),OAM(1),(RABC(I),H(I),NRD(I),(EVALE(J,I),J=1,NDIM),I=
     11,ISMAX),TIMDAT(1:80),(LAMSQE(I),I=1,NDIM),ILRS,((EFCNM1(I,J),
     2I=1,NDIM),J=1,NDIM)
      CLOSE(UNIT=FU14)
      END
